local lg = love.graphics

--require("utils.pure.astar")
local newHud = require("core.hud")

local kinds =
{
  [0x000000] = "empty",
  [0x555555] = "block",
  -- environment
  [0xffffff] = "wall",
  [0xc0c0c0] = "wallb",
  [0x808080] = "platform",
  [0x404040] = "ladder",
  [0xff0000] = "exploding",
  [0x505050] = "conveyora",
  [0x606060] = "conveyorb",
  [0x707070] = "spikes",
  [0x303030] = "collapsing",
  [0xff8000] = "breakable",
  [0x808000] = "switch",
  [0xff8080] = "lava",
  [0xffaaff] = "bonus",
  [0xffaaaa] = "heart",
  -- enemies
  [0x800000] = "droplet",
  [0x00ffff] = "landmine",
  [0x0000ff] = "spewer",
  [0xcc00ff] = "crawler",
  [0xff00ff] = "gunman",
  [0xaa00aa] = "gunmanb",
  [0x770077] = "gunmanc",
  [0x220022] = "gunmand",
  [0x00ddff] = "shield",
  [0x800040] = "ram",
  [0x900050] = "rhino",
  [0xffff00] = "tornado",
  [0xaaff00] = "spike",
  [0x2020ff] = "column",
  [0xffaa00] = "rocket",
  [0xccaaff] = "jellyfish",
  [0xff0080] = "hopper",
  [0xcc0070] = "hopperb",
  [0xaaaaff] = "octopus",
  [0x00ff80] = "crusher",
  [0x008080] = "bird",
  [0xc080c0] = "skullmaster",
  [0xc080c1] = "skullmasterb",
  [0xc080c2] = "skullmasterc",
}

local nonwalkable =
{
  base = true,
  --block = true,
  wall = true,
  wallb = true,
  exploding = true,
  conveyora = true,
  conveyorb = true,
  collapsing = true,
  breakable = true,
  lava = true,
}

return function(session)
  print("loading:"..session.level)
  local cat = game.cat
  local title = session.level
  local data = love.image.newImageData("lvl/"..title..".png")
  local w, h = data:getDimensions()

  local def = {}
  if app.exists("lvl/"..title..".lua") then
    local rn = "lvl."..title
    package.loaded[rn] = nil
    def = require(rn)
  end
  
  local lvl = {}
  lvl.kinds = kinds
  lvl.session = session
  lvl.title = title
  lvl.view = love.scene.newView()
  if def.bg then
    lvl.view:setBackgroundRGB(def.bg)
  end
  lvl.data =
  {
    tick = 0,
    elapsed = 0,
    score = 0,
    p1a = {}, p2a = {},
    over = false,
    complete = false,
    controlled = 0,
    finished = 0,
    sent = {},
    received = {},
    sync = 0,
    resend = 0,
  }
  
  lvl.layers = {}
  for i = 1, 10 do
    lvl.layers[i] = lvl.view:newLayer(0, 0)
  end
  lvl.mesh = {}
  lvl.bg = lvl.layers[1]:newSprite()

  lvl.hud = newHud(lvl)
  lvl.obj = {}
  lvl.cells = {}
  lvl.dynamic = {}
  lvl.range = {}
  lvl.map = {}
  lvl.destroyed = {}
  
  lvl.def = def
  lvl.width = w
  lvl.height = h
  
  lvl.world = love.physics.newWorld()
  lvl.world:setSleepingAllowed(true)
  lvl.world:setGravity(0, 400)

  -- tiles
  def.hidden = def.hidden or {}
  local pts = 0
  for y = 1, h do
    for x = 1, w do
      local r, g, b, a = data:getPixel(x - 1, y - 1)
      local rgb = math.floor(b*255) + math.floor(g*255*0x100) + math.floor(r*255*0x10000)
      local kind = kinds[rgb]
      if not kind then
        error(x..","..y)
      end
      local i = (y - 1)*w + x
      lvl.map[i] = kind
      local c = game.obj[kind]
      if c then
        pts = pts + (c.bonus or 0)
        if kind == "heart" then
          table.insert(def.hidden, {x - 1, y  - 1})
        end
      end
    end
  end
  lvl.data.maximum = pts
  
  lvl.world:setCallbacks(nil, nil, function(f1, f2, contact)
    local b1 = f1:getBody()
    local b2 = f2:getBody()
    -- agent vs environment
    local c1 = f1:getFilterData()
    local c2 = f2:getFilterData()
    -- align agent first
    if c2 ~= 0 and c1 > c2 then
      b1, b2 = b2, b1
      c1, c2 = c2, c1
    end
    if c1 >= cat.player and c1 <= cat.enemy then
      -- one-sided platforms
      if c2 == cat.platform then
        local nx, ny = contact:getNormal()
        local vx, vy = b1:getLinearVelocity()
        if ny >= 0 or vy < -1 then
          contact:setEnabled(false)
          return
        end
      end
    end
    local u1 = b1:getUserData()
    local u2 = b2:getUserData()
    if u1:call("collision", u2, contact) ~= false then
      u2:call("collision", u1, contact)
    end
  end)

  function lvl.destroy()
    lvl.destroyQueue()
    for i = #lvl.obj, 1, -1 do
      lvl.obj[i]:destroy()
      lvl.obj[i] = nil
    end
    lvl.obj = nil
    for i = #lvl.layers, 1, -1 do
      lvl.layers[i]:destroy()
      lvl.layers[i] = nil
    end
    lvl.layers = nil
    lvl.view:destroy()
    lvl.view = nil
    lvl.world:setCallbacks()
    lvl.world:destroy()
    lvl.world = nil  
    lvl.data = nil
    lvl.cells = nil
    lvl.dynamic = nil
    lvl.destroyed = nil
    lvl.map = nil
    lvl.def = nil
    lvl.width = nil
    lvl.height = nil
    lvl.mesh = nil
  end

  function lvl.getDimensions()
    return w, h
  end
  
  function lvl.isWalkable(index)
    local kind = lvl.map[index]
    return nonwalkable[kind] ~= true
  end
  
  function lvl.isWalkableXY(x, y)
    if x >= 1 and x <= w and y >= 1 and y <= h then
      local index = (y - 1)*w + x
      return lvl.isWalkable(index)
    end
  end
  
  function lvl.getTile(x, y)
    if x >= 1 and x <= w and y >= 1 and y <= h then
      local index = (y - 1)*w + x
      return lvl.map[index]
    end
  end
  
  function lvl.updateRange(p)
    if not p then
      return
    end
    local x, y = p:getPosition()
    local cx = math.floor(x/8 + 0.5)
    local cy = math.floor(y/8 + 0.5)
    local r1 = lvl.range[p.name]
    local r2 = { cx - 48, cy - 24, cx + 48, cy + 24 }
    lvl.changeRange(r2, r1)
    lvl.range[p.name] = r2
  end

  function lvl.changeRange(r2, r1)
    if r1 and r1[1] == r2[1] and r1[2] == r2[2] and r1[3] == r2[3] and r1[4] == r2[4] then
      return
    end
    local x1, y1, x2, y2 = unpack(r2)
    for x = x1, x2 do
      for y = y1, y2 do
        local i = (y - 1)*w + x
        local obj = lvl.cells[i]
        if obj then
          obj:ref()
        end
      end
    end
    if r1 then
      local x3, y3, x4, y4 = unpack(r1)
      for x = x3, x4 do
        for y = y3, y4 do
          local i = (y - 1)*w + x
          local obj = lvl.cells[i]
          if obj then
            obj:unref()
          end
        end
      end
    end
  end
  
  function lvl.call(f, ...)
    if type(def[f]) == "function" then
      def[f](lvl, ...)
    end
  end

  function lvl.broadcast(f, ...)
    for i, obj in ipairs(lvl.obj) do
      local n = obj.refs
      if n == nil or n > 0 or obj.nosleep then
        obj:call(f, ...)
      end
    end
    lvl.call(f, ...)
  end
  
  function lvl.adjacent(a, b)
    if a and b then
      local i1 = a.id
      local i2 = b.id
      if lvl.isWalkable(i2, true) then
        if i1 == i2 - 1 or i1 == i2 + 1 or i1 == i2 - w or i1 == i2 + w then
          return true
        end
      end
    end
    return false
  end

  function lvl.newObject(n, ...)
    assert(game.obj[n], n)
    local obj = game.oo.instance(game.obj[n])
    obj.type = n
    obj:init(lvl, ...)
    if not obj:isStatic() then
      --lvl.dynamic[obj] = true
      table.insert(lvl.dynamic, obj)
    end
    table.insert(lvl.obj, obj)
    return obj
  end

  function lvl.destroyObject(obj)
    lvl.destroyed[obj] = true
  end
  
  function lvl.newCell(x, y)
    assert(x >= 0 and x <= w and y >= 0 and y <= h)
    local i = (y - 1)*w + x
    local n = lvl.map[i]
    if n == "empty" or n == "block" then
      return
    end
    local bits = 0
    if y < h and lvl.map[i + w] ~= n then
      bits = bits + 1 -- up
    end
    if y > 1 and lvl.map[i - w] ~= n then
      bits = bits + 2 -- down
    end
    if x > 1 and lvl.map[i - 1] ~= n then
      bits = bits + 4 -- left
    end
    if x < w and lvl.map[i + 1] ~= n then
      bits = bits + 8 -- right
    end
    
    local wx, wy = x*8, y*8
    if n == "wall" then
      lg.rectMesh(wx - 4, wy - 4, 8, 8, lvl.mesh)
      if bits == 0 then
        return
      end
    end
    
    assert(game.obj[n], n)
    local obj = game.oo.instance(game.obj[n])
    obj.type = n
    obj.index = i
    obj.bits = bits
    obj:init(lvl, wx - 4, wy - 4, bits)
    if obj:isStatic() then
      lvl.cells[i] = obj
    else
      --lvl.dynamic[obj] = true
      table.insert(lvl.dynamic, obj)
    end
    table.insert(lvl.obj, obj)
    return obj
  end
  
  function lvl.destroyCell(obj)
    local i = obj.index
    local x = i%w
    local y = (i - x)/w
    x = x - 1
    assert(x >= 0 and x <= w and y >= 0 and y <= h)
    local obj = lvl.cells[i]
    if obj then
      lvl.map[i] = 0
      lvl.cells[i] = nil
      lvl.destroyObject(obj)
    end
  end

  function lvl.destroyQueue()
    local list = lvl.obj
    for i = #list, 1, -1 do
      local obj = list[i]
      if lvl.destroyed[obj] then
        lvl.destroyed[obj] = nil
        for i, o in ipairs(lvl.dynamic) do
          if o == obj then
            table.remove(lvl.dynamic, i)
            break
          end
        end
        --lvl.dynamic[obj] = nil
        obj:destroy()
        table.remove(list, i)
      end
    end
  end
  
  function lvl.flood(i, list, cache)
    list = list or {}
    local t = lvl.map[i]
    if not cache then
      cache = { [t] = true }
    end
    if cache[t] and not cache[i] then
      cache[i] = true
      table.insert(list, i)
      lvl.flood(i - 1, list, cache)
      lvl.flood(i + 1, list, cache)
      lvl.flood(i - w, list, cache)
      lvl.flood(i + w, list, cache)
    end
    return list, cache
  end
  
  function lvl.draw()
    lvl.broadcast("sync")

    local w, h = app.w, app.h
    local rh = 20*8
    local rw = math.min(w/app.s, 21/9*rh)
    local view = lvl.view
    view:setShader(app.shaders.default)
    view:setScale(app.s, app.s)
    
    local d = lvl.data
    if d.p1 and d.p2 then
      local x1, y1 = d.p1:getPosition()
      local x2, y2 = d.p2:getPosition()
      local dx = math.abs(x2 - x1)
      local dy = math.abs(y2 - y1)
      if dx > rw/2 - game.clampx or dy > rh/2 - game.clampy then
        -- split screen
        if d.screen ~= "split" then
          d.screen = "split"
          d.flipped = x1 > x2
        end
        if d.flipped then
          x1, y1, x2, y2 = x2, y2, x1, y1
        end
        d.cx, d.cy = game.clampCamera(lvl, rw/2, rh, x1, y1, d.cx, d.cy)
        d.cx2, d.cy2 = game.clampCamera(lvl, rw/2, rh, x2, y2, d.cx2, d.cy2)
        game.drawView(lvl, 0, 0, w/2, h, d.cx, d.cy)
        game.drawView(lvl, w/2, 0, w/2, h, d.cx2, d.cy2)
        lg.setColor(1, 1, 1, 1)
        lg.reset()
        lg.setLineWidth(app.s*2)
        lg.line(w/2, 0, w/2, h)
      else
        -- shared screen
        d.screen = "shared"
        d.cx, d.cy = game.clampCamera(lvl, rw, rh, x1, y1, d.cx, d.cy)
        d.cx2, d.cy2 = nil, nil
        game.drawView(lvl, 0, 0, w, h, d.cx, d.cy)
      end
    else
      -- single screen
      d.screen = "single"
      local x, y = d.cx, d.cy
      local p = d.p1 or d.p2
      if p then 
        x, y = p:getPosition()
      end
      d.cx, d.cy = game.clampCamera(lvl, rw, rh, x, y, d.cx, d.cy)
      d.cx2, d.cy2 = nil, nil
      game.drawView(lvl, 0, 0, w, h, d.cx, d.cy)
    end

    -- hud
    lvl.hud.draw(w, h, rw, rh)
  end
  
  function lvl.init()
    print("initializing:"..lvl.title)
    
    local border = lvl.newObject("body", 0, 0, 1)
    local body = border.body
    body:newEdge(0, 0, w*8, 0)
    body:newEdge(0, 0, 0, h*8)
    body:newEdge(w*8, 0, w*8, h*8)
    body:newEdge(0, h*8, w*8, h*8)
    body:setFilter(cat.wall, cat.default, 0)

    for x = 1, w do
      for y = 1, h do
        lvl.newCell(x, y)
      end
    end

    -- moving
    if def.moving then
      for _, v in ipairs(def.moving) do
        for j, pt in ipairs(v.tiles) do
          local i = pt[2]*w + pt[1] + 1
          local obj = lvl.cells[i]
          assert(obj, pt[1]..','..pt[2])
          lvl.cells[i] = nil
          obj:setMoving(v.path, v.speed)
        end
      end
    end

    -- disappearing
    if def.disappearing then
      for _, v in ipairs(def.disappearing) do
        local sensor = lvl.world:newBody(0, 0)
        for j, pt in ipairs(v) do
          local i = pt[2]*w + pt[1] + 1
          local obj = lvl.cells[i]
          assert(obj, pt[1]..','..pt[2])
          lvl.cells[i] = nil
          obj:setDisappearing(v.period, v.dutycycle, v.time)
          obj:setSensor(sensor)
        end
      end
    end
    
    -- sparks
    if def.sparks then
      for i, v in ipairs(def.sparks) do
        local tx, ty = unpack(v)
        lvl.newObject("spark", tx*8, ty*8, v.rev)
      end
    end
    
    -- hidden areas
    if def.hidden then
      for _, pt in ipairs(def.hidden) do
        local tx, ty = unpack(pt)
        lvl.newObject("hidden", tx, ty)
      end
    end

    -- switches
    if def.switches then
      for i, v in ipairs(def.switches) do
        local sensor = lvl.world:newBody(0, 0)
        for j, pt in ipairs(v.tiles) do
          local index = pt[2]*w + pt[1] + 1
          local obj = lvl.cells[index]
          assert(obj, pt[1]..','..pt[2])
          v.tiles[j] = obj
          obj:setSensor(sensor)
        end
        for j, pt in ipairs(v.points) do
          local index = pt[2]*w + pt[1] + 1
          local obj = lvl.cells[index]
          assert(obj, pt[1]..','..pt[2])
          v.points[j] = obj
        end
        for _, obj in ipairs(v.points) do
          local on = (v.state == true or v.state == nil)
          obj:setTarget(v.tiles, v.points)
          obj:toggle(on)
          obj.limit = v.limit
          for i, obj in ipairs(v.tiles) do
            obj:toggle(on)
          end
        end
      end
    end

    -- beam
    if def.mounts then
      for _, v in ipairs(def.mounts) do
        v[1], v[2] = v[1]*8 + 4, v[2]*8 + 4
        local obj = lvl.newObject("mount", unpack(v))
        obj:setPeriod(v.period, v.cycle, v.time)
      end
    end

    --for i, obj in pairs(lvl.cells) do
      --obj:unref()
    --end
    for i = 1, w*h do
      local obj = lvl.cells[i]
      if obj then
        obj:unref()
      end
    end
    
    -- checkpoints
    if def.checkpoint then
      lvl.checkpoint = {}
      for i, v in ipairs(def.checkpoint) do
        local x, y = unpack(v)
        local wx, wy = x*8 + 4, y*8 + 4
        if not v.kind then
          v.kind = i < #def.checkpoint and "right" or "hidden"
        end
        local obj = lvl.newObject("checkpoint", wx, wy, v.kind, i)
        if i == 1 then
          for j = 1, session.joined do
            obj:join("p"..j)
          end
        end
        lvl.checkpoint[i] = obj
      end
    end

    local mw, mh = w*8, h*8
    local ww = 8*32, 8*32
    
    lg.rectMesh(-ww + 4, 4, ww, mh, lvl.mesh)
    lg.rectMesh(mw + 4, 4, ww, mh, lvl.mesh)
    
    local mesh = lg.newMesh(lvl.mesh, "triangles", "static")
    lvl.bg:setColorRGB(0, 0, 0)
    lvl.bg:setGraphic(mesh, -4, -4)
    
    local n = table.random({ "roundone", "letsgo", "start" })
    app.say(n, 5)
    
    lvl.call("init")
    
    print("ready")
  end
  
  return lvl
end