local progress = {}

progress.data = require("defs.stats")

function progress.load(t)
  print("loading progress")
  -- load
  local fn = app.ops.version.."/"..app.ops.user..".sav"
  local u = app.load(fn)
  if type(u) ~= 'table' then
    print("checking for legacy progress")
    u = require("core.legacy")
  else
    print("loading progress:"..fn)
  end
  -- replace
  if type(u) == "table" then
    for k, v in pairs(t) do
      local w = u[k]
      if type(v) == "table" and type(w) == "string" then
        w = app.loadstring("return"..w)
      end
      if type(v) == type(w) then
        t[k] = w
      end
    end
  end
  return t
end

function progress.save(t)
  t.modified = os.date("%Y-%m-%dT%H:%M:%S")
  local fn = app.ops.version.."/"..app.ops.user..".sav"
  if not app.exists(fn) then
    t.created = t.modified
  end
  local out = "return {\n"
  for n, v in pairs(t) do
    if type(v) == "string" then
      v = string.format("%q", v)
    elseif type(v) == "table" then
      local q = {}
      for k, w in pairs(v) do
        if type(w) == "string" then
          w = string.format("%q", w)
        end
        w = tostring(w)
        table.insert(q, '["'..k..'"]='..w)
      end
      v = "'{"..table.concat(q,",").."}'"
    else
      v = tostring(v)
    end
    out = out..n.."="..v..",\n"
  end
  out = out.."}"
  -- assumes the directory exists, will fail otherwise
  print("saving progress:"..fn)
  if not app.save(fn, out) then
    print("saving progress failed:"..fn)
    return false
  end
  return true
end

function progress.import()
  progress.load(progress.data)
end

function progress.export()
  progress.save(progress.data)
end

function progress.receive(user)
  local d = progress.data
  for k, v in pairs(d) do
    if type(v) == "number" then
      local _v = user:getStat(k)
      if _v and _v > v then
        d[k] = _v
      end
    elseif type(v) == "boolean" then
      local _v = user:getAchievement(k)
      if _v and _v == true then
        d[k] = _v
      end
    end
  end
end

function progress.send(user)
  -- portal/steam stats
  local d = progress.data
  for k, v in pairs(d) do
    if type(v) == "number" then
      v = math.floor(v)
      user:setStat(k, v)
    elseif type(v) == "boolean" then
      if v == true then
        user:setAchievement(k)
      end
    end
  end
  user:storeStats()
end

return progress