local layout =
{
  title = app.lang.levelselect,
  cols = 3,
  rows = 3,
}

local p = game.progress.data
local files = app.getdir("/lvl/", "lua")
local levels = {}
for i, file in ipairs(files) do
  local n = file:match("(%d+)%.lua")
  table.insert(levels, n)
end
table.sort(levels)
local completed = 0
for i, n in ipairs(levels) do
  local t = p["time"..n] or 0
  if t ~= 0 then
    completed = i
  end
end
for i, n in ipairs(levels) do
  local txt = string.format("%s %02d", app.lang.leveln, n)
  if app.exists("/lvl/"..n..".png") then
    local lvl = game.newLevel({ level = n })
    local score = string.format("💀%05d/%05d", p["score"..n] or 0, lvl.data.maximum)
    local seconds = app.lang.seconds(p["time"..n] or 0)
    local timen = string.format("🕒%s", seconds)
    txt = txt.."\n\n"..timen.."\n"..score
  end
  local item = { text = txt, value = n, disabled = i > completed + 1 }
  table.insert(layout, item)
end

local session
function layout.init(s)
  session = s
  for i, n in ipairs(levels) do
    if n == session.level then
      layout.selected = i
      break
    end
  end
  if session.level == "1" then
    app.say("choose")
  end
end

function layout.cancel()
  app.popstate()
  app.pushstate("states.title")
end

function layout.choose(i)
  local n = layout[i].value
  app.popstate()
  if not app.exists("/lvl/"..n..".png") then
    app.pushstate("states.buy")
  else
    session.level = n
    app.pushstate("states.play", session)
  end
end

function layout.update(dt)

end

return layout