if not app.arg.devmode then
  function love.errorhandler(msg)
    local trace = debug.traceback(tostring(msg), 3)
    local report = trace:gsub("\n[^\n]+$", "")
    app.error(report)
    app.report(app.ops.build)
  end
end

function love.load()
  app.sync = 0
  app.tick = 0
  if app.ops.display > love.window.getDisplayCount() then
    print("adjusting display")
    app.ops.display = 1
  end
  local w, h = love.graphics.getDimensions()
  local dw, dh = love.window.getDesktopDimensions(app.ops.display)
  if not app.ops.fullscreen and dh - h < 25 then
    print("adjusting window")
    love.window.maximize()
  end
  love.resize(w, h)
  app.pushstate("states.splash")
end

function love.update(dt)
  app.tick = app.tick + 1
  app.callstate("update", dt)
  if app.input then
    app.input.poll()
  end
  if app.portal then
    app.portal.update()
  end
  if app.profile then
    if app.tick%100 == 0 then
      print("\n"..app.profile.report(20))
      app.profile.reset()
    end
  end
end

function love.focus(on)
  -- mute when out of focus
  if love.audio then
    love.audio.setVolume(on and 1 or 0)
  end
end

function love.draw()
  local t1 = app.sync
  local t2 = love.timer.getTime()
  app.sync = t2
  app.callstate("draw", t2 - t1)
  local s = app.shaders and app.shaders.default
  if s and s ~= app.shaders.none then
    if app.ops.shader == "crt" then
      s:send("scale", app.s)
      s:send("time", os.clock())
    elseif app.ops.shader == "apple" then
      s:send("scale", app.s)
    end
  end
end

function love.resize(w, h)
  w = w - w%2
  h = h - h%2
  app.w, app.h = w, h
  local minh = 20*8
  local s = h/minh
  if s > 1 and not app.ops.stretch then
    s = math.floor(s)
  end
  if s > 1 and w < (minh/3*4)*s then
    s = s/2
  end
  app.s = s
  app.callstate("resize", w, h)
end

function love.quit()
  print("quitting")
  while app.popstate() do end
  local ops = app.ops
  if not ops.fullscreen then
    ops.x, ops.y = love.window.getPosition()
    ops.width, ops.height = love.window.getMode()
  else
    ops.width, ops.height = 0, 0
  end
  print("saving options")
  local list = {}
  for id, n in pairs(app.ops.nullzone) do
    local kv = string.format("[%q]=%s", id, n)
    table.insert(list, kv)
  end
  app.ops.nullzone = "{"..table.concat(list, ",").."}"
  app.saveoptions(ops.version.."/options.ser", ops)
  app.release()
end