local parent = require("obj.fsm")
local class = game.oo.class(parent)

local cat = game.cat

function class:init(lvl, x, y, d)
  parent.init(self, lvl, x, y, d or 5)
  self.body:setType("dynamic")
  self.moving = false
  self.grounded = false
  self.lastgrounded = -1
  self.elapsed = 0
  self.spawns = 0
end

function class:destroy()
  self.moving = nil
  self.grounded = nil
  self.lastgrounded = nil
  self.elapsed = nil
  self.spawns = nil
  parent.destroy(self)
end

function class:dropBonus(dx, dy)
  if self.spawns <= 1 then
    dx = dx or 0
    dy = dy or -8
    self:spawn("score", dx, dy, self.bonus)
    app.playSound("powerup")
  end
end

function class:getMovement()
  return 0, 0, false, false
end

function class:canJump(dx, dy)
  dx, dy = dx*8, dy*8
  local ox, oy = self:getPosition()
  local sx, sy = self:getScale()
  local up = self:rayCast("env", ox, oy, ox, oy + dy)
  return not (up or self:rayCast("env", ox, oy + dy, ox + dx*sx, oy + dy))
end

function class:isBlocked(x1, y1, x2, y2)
  x1, y1 = x1*8, y1*8
  x2, y2 = x2*8, y2*8
  local ox, oy = self:getPosition()
  local sx, sy = self:getScale()
  return self:rayCast("env", ox + x1*sx, oy + y1, ox + x2*sx, oy + y2)
end

function class:lineOfSight(dx, dy)
  local ox, oy = self:getPosition()
  local sx, sy = self:getScale()
  local q, qx, qy = self:rayCast("player", ox, oy, ox + sx*dx*8, oy + dy*8)
  if q and not self:rayCast("wall", ox, oy, qx, qy) then
    return q, qx, qy
  end
end

function class:update(dt)
  parent.update(self, dt)
  -- current movement, velocity and displacement
  local x, y = self:getPosition()
  local xm, ym, jump, shoot = self:getMovement()
  local xv, yv = self:getVelocity()
  local d = self.lvl.data
  local gs = self.body:getGravityScale()
  local g = (gs > 0) and 1 or -1
  local isonwall = self:isPushedOnAxis(0, g, cat.wall)
  local isonplatform = self:isPushedOnAxis(0, g, cat.platform)
  local isgrounded = isonwall or isonplatform
  local wasgrounded = self.grounded
  self.grounded = isgrounded
  if isgrounded and not wasgrounded then
    self:message("hitground", self.yv or yv)
  elseif not isgrounded and wasgrounded then
    if not jump then
      self.lastgrounded = d.elapsed
    end
    self:message("fall")
  end
  if self:isPushedOnAxis(0, -g, cat.wall) and yv > 0 then
    self:message("hitroof", self.yv or yv)
  end
  self.xv, self.yv = xv, yv
  self.candropdown = false
  if not isonwall then
    if isonplatform then
      self.candropdown = true
    end
  end
  if jump then
    self:message("jump")
  end
  if shoot then
    self:message("shoot")
  end
end

return class