local lg = love.graphics

local parent = require("obj.agent")
local class = game.oo.class(parent)

function class:init(lvl, x, y, nx, ny)
  parent.init(self, lvl, x, y, 6)
  self.body:setType("kinematic")

  self.nx = nx
  self.ny = ny
  self.length = 0
  self.width = 0
  self.maxwidth = 8 - 2
  self.on = false

  local a = math.atan2(ny, nx)
  local sprite = self.sprite
  sprite:setRotation(a)
  local sprite_draw = sprite.draw
  sprite.draw = function(sprite)
    lg.push()
    lg.setColor(1, 0, 0, 1)
    if lvl.data.elapsed%0.2 > 0.1 then
      lg.setColor(1, 1, 1, 1)
    end
    if self.x2 and self.y2 and self.width > 0 then
      lg.setLineWidth(self.width)
      lg.line(self.x1, self.y1, self.x2, self.y2)
    end
    lg.pop()
    sprite_draw(sprite)
  end
  
  self:setMode(true)
end

function class:destroy()
  self.on = nil
  self.nx = nil
  self.ny = nil
  self.length = nil
  self.width = nil
  parent.destroy(self)
end

function class:setMode(on, over)
  if on ~= self.on and self.body then
    self:playSound(on and "laser2" or "laser")
    self.on = (on == true)
    self.over = (on == false and over == true)
  end
end

function class:getMode(on)
  return self.on
end

function class:interrupt()
  self:setMode(false, true)
end

function class:update(dt)
  parent.update(self, dt)
  if self.on then
    local n = self.length
    self.length = math.min(n + 400*dt, 8*30)
    self.width = self.maxwidth
    local q = 4 - 0.1
    local x, y = self:getPosition()
    local nx, ny = self.nx, self.ny
    self.x1, self.y1 = x - nx*q, y - ny*q
    self.x2, self.y2 = x + nx*n, y + ny*n

    local w = self.width
    local n = math.ceil(w/2)
    for i = 0, n do
      local d = (i/n)*2 - 1
      d = d*(w/2)
      local dx, dy = ny*d, nx*d
      local x1, y1 = self.x1 + dx, self.y1 + dy
      local x2, y2 = self.x2 + dx, self.y2 + dy
      if n > 1 then
        local other, qx, qy = self:rayCast("laser", x1, y1, x2, y2)
        if other then
          other:call("shot", 1)
          local vx, vy = qx - x1, qy - y1
          local v = math.sqrt(vx*vx + vy*vy)
          self.x2, self.y2 = nx*v + self.x1, ny*v + self.y1
        end
      end
    end
  else
    self.width = math.max(self.width - 50*dt, 0)
    if self.width <= 0 then
      self.length = 0
      if self.over then
        self.lvl.destroyObject(self)
      end
    end
  end
end

return class