local cat = game.cat
local states = {}

states.spawn = {}
function states.spawn:update(obj, dt)
  obj.body:setActive(true)
  obj:reposition()
  obj.spawns = obj.spawns + 1
  obj.health = 1
  obj.invincible = 0
  obj:setState("flying")
  obj:setScale(-1, 1)
end

states.flying = {}
function states.flying:enter(obj)
  obj.elapsed = math.random()*2
  obj:loopSound("flap")
end
function states.flying:exit(obj)
  obj:stopLoop("flap")
end
function states.flying:update(obj, dt)
  obj:updateSound(obj.loop)
  local a = math.sin(obj.elapsed%2/2*math.pi*2)
  local x, y = obj:getPosition()
  local sx, sy = obj:getScale()
  local dx, dy = obj.speed*dt*sx, a*(8*2)
  obj:setPosition(x + dx, obj.iy + dy)
  local tx, ty = obj:getTile(x + dx*2, obj.iy)
  local ahead = obj.lvl.getTile(tx, ty)
  if ahead == nil or ahead == "block" then
    obj:setScale(-sx, sy)
    local d = obj.lvl.data
    local pc = d.p1 or d.p2
    if pc then
      local x2, y2 = pc:getPosition()
      if math.random(2) == 1 and y2 > y and y2 - y < 8*20 then
        obj:setState("diving")
      end
    end
  end
end

states.diving = {}
function states.diving:enter(obj)
  obj.elapsed = math.random()*2
  obj:loopSound("rocket")
end
function states.diving:exit(obj)
  obj:stopLoop("rocket")
end
function states.diving:update(obj, dt)
  obj:updateSound(obj.loop)
  local x, y = obj:getPosition()
  local sx, sy = obj:getScale()
  local dx, dy = dt*obj.speed*2*sx, dt*obj.speed*2
  obj:setPosition(x + dx, y + dy)
  if obj.lvl.data.tick%3 == 0 then
    local n = "trail"..math.random(1, 3)
    local anim = obj:spawn("animation", 0, -2, 9, n)
    anim:setColor(128, 128, 128)
  end
  if not obj:inRange() and not obj:inRange(obj.ix, obj.iy) then
    obj:setScale(-sx, sy)
    obj:reposition()
    obj:setState("flying")
  end
end

states.exploding = {}
function states.exploding:update(obj, dt)
  obj:debree(8, 0, 0, 255, 0, 0)
  obj:spawn("explosion", 0, 0, 0, 8*3, 0.25)
  obj:setGraphic()
  obj:setState("dead")
end

states.dead = {}
function states.dead:enter(obj)
  local list = {}
  obj:detect(nil, list)
  for i, other in ipairs(list) do
    other:call("damage", 1)
  end
  
  obj.body:setActive(false)
end
function states.dead:awake(obj, dt)
  obj:setState("spawn")
end

return states