local cat = game.cat
local animations =
{
  file = "bird", ox = -12, oy = -16,
  flying = { start = 1, frames = 7, cycle = "loop", steps = 4 },
  diving = { start = 8, frames = 1, cycle = "none" },
}

local parent = require("obj.agent")
local class = game.oo.class(parent)

class.states = require("obj.bird-states")
class.animations = animations
class.bonus = 500
class.speed = 50
class.nosleep = true

function class:init(lvl, x, y, d)
  parent.init(self, lvl, x, y, 10)
  local body = self.body
  body:setGravityScale(0)
  self.shape = body:newCircle(3)
  self.shape:setFilterData(cat.enemy, cat.player + cat.bullet, -2)
  self.shape:setFriction(0)
  self.sensor = body:newCircle(8*3.75)
  self.sensor:setFilterData(cat.enemy, cat.default, -2)
  self.sensor:setSensor(true)
  self:setState("spawn")
  local sx = (math.random(1, 2) == 1) and -1 or 1
  self:setScale(sx, 1)
end

function class:destroy()
  self:setState()
  self.shape = nil
  self.speed = nil
  --self.turned = nil
  parent.destroy(self)
end

function class:onDie(pts)
  self:dropBonus()
  self:setState("exploding")
end

function class:onDamage(h)
  game.screenshake(self.lvl, 0.25)
end

function class:collision(other, contact)
  if other:getType() == "player" then
    other:call("hit", 1)
    if math.random(3) == 1 then
      app.say("chicken", 6)
    end
    if self.health > 0 then
      self:damage()
    end
  end
  contact:setEnabled(false)
end

function class:shot(pts)
  self:damage(pts)
end

return class