local parent = require("obj.base")
local class = game.oo.class(parent)

function class:init(lvl, x, y, z)
  parent.init(self, lvl, x, y, z)
  local body = lvl.world:newBody(x, y, "static")
  body:setFixedRotation(true)
  body:setUserData(self)
  self.body = body
end

function class:destroy()
  self.body:destroy()
  self.body = nil
  parent.destroy(self)
end

function class:isStatic()
  return self.body:getType() == "static"
end

function class:getPosition()
  return self.body:getPosition()
end

function class:setPosition(x, y)
  return self.body:setPosition(x, y)
end

function class:getVelocity()
  return self.body:getLinearVelocity()
end

function class:setVelocity(vx, vy)
  return self.body:setLinearVelocity(vx, vy)
end

function class:setAwake(on)
  if self.nosleep then
    return
  end
  on = on and self.mode or false
  self.sprite:setVisible(on)
  self.body:setAwake(on)
end

function class:toggle(on)
  self.mode = on
  self.sprite:setVisible(on)
  self.body:setActive(on)
end

function class:isPushedOnAxis(ax, ay, cat)
  local contacts = self.body:getContacts()
  for i, c in ipairs(contacts) do
    if c:isTouching() and c:isEnabled() then
      local x1, y1 = c:getPositions()
      if x1 and y1 then
        local nx, ny = c:getNormal()
        local f1, f2 = c:getFixtures()
        local other = f1:getBody()
        if other ~= self then
          nx, ny = -nx, -ny
          f1, f2 = f2, f1
        end
        if ax*nx + ay*ny > 0 then
          if cat == nil or cat == f2:getFilterData() then
            return true
          end
        end
      end
    end
  end
  return false
end

function class:movePlatform(x2, y2, fixture)
  local body = self.body
  local x1, y1 = body:getPosition()
  local dx, dy = x2 - x1, y2 - y1

  if fixture then
    local tick = self.lvl.data.tick
    local contacts = body:getContacts()
    for _, contact in ipairs(contacts) do
      if contact:isTouching() and contact:isEnabled() then
        local f1, f2 = contact:getFixtures()
        local nx, ny = contact:getNormal()
        local b1, b2 = f1:getBody(), f2:getBody()
        if b2 == body then
          b1, b2 = b2, b1
          nx, ny = -nx, -ny
        end
        if (f1 == fixture or f2 == fixture) and (ny < 0) then
          local other = b2
          local user = other:getUserData()
          if (user.moved or 0) < tick then
            user.moved = tick
            local x3, y3 = other:getPosition()
            other:setPosition(x3 + dx, y3 + dy)
          end
        else
          contact:setRestitution(1)
          contact:setFriction(0)
        end
      end
    end
  end

  body:setPosition(x2, y2)
  local x3 = math.floor(x2 + 0.5)
  local y3 = math.floor(y2 + 0.5)
  self.sprite:setPosition(x3, y3)
end

function class:sync()
  parent.sync(self)
  local x, y = self.body:getPosition()
  local rx = math.floor(x + 0.5)
  local ry = math.floor(y + 0.5)
  self.sprite:setPosition(rx, ry)
end

return class