local parent = require("obj.body")
local class = game.oo.class(parent)
local cat = game.cat

class.speed = 150
class.nosleep = true
class.width = 2
class.height = 0.5
class.graphic = "bullet"

function class:init(lvl, x, y, nx, ny, mask)
  mask = mask or cat.player
  parent.init(self, lvl, x, y, 8)
  local a = math.atan2(ny, nx)
  local s = self.speed
  local body = self.body
  body:setType("dynamic")
  body:setGravityScale(0)
  body:setLinearVelocity(s*nx, s*ny)
  body:setAngle(a)
  body:setBullet(true)
  self.shape = body:newBox(self.width, self.height)
  self.shape:setFilterData(cat.bullet, cat.wall + mask, -3)
  self:setGraphic(self.graphic)
  self:playSound("shoot")
  self.dead = false
  local flash = self:spawn("animation", 2*nx, 0, 8, "spark1")
  flash:setColor(255, 255, 0)
end

function class:destroy()
  self.shape = nil
  self.dead = nil
  parent.destroy(self)
end

function class:update(dt)
  local x, y = self:getPosition()
  self.sprite:setPosition(x, y)
  if self.dead then
    self:playSound("pop")
    self.lvl.destroyObject(self)
    local obj = self:spawn("animation", 0, 0, 8, "spark2")
    obj:setColor(255, 255, 0)
  end
end

function class:collision(other, contact)
  if not self.dead then
    local a, b = other:call("shot", 1, self, contact)
    self.dead = (b == nil)
    if self.dead and math.random(8) == 1 then
      local t = other:getType()
      if t == "player" then
        local q = table.random({ "triggered", "busted", "mandown" })
        app.say(q, 6)
      elseif t == "gunman" or t == "spewer" or t == "crawler" then
        local q = table.random({ "betweeneyes", "dropdead", "headshot", "staydead" })
        app.say(q, 6)
      end
    end
  end
end

return class