local cat = game.cat
local parent = require("obj.trigger")
local class = game.oo.class(parent)

function class:init(lvl, x, y, kind, ident)
  parent.init(self, lvl, x, y, 5)
  self.ident = ident
  self.kind = kind
  if self.kind == "left" then
    self:setScale(-1, 1)
  end
  self.shape = self.body:newBox(8/4, 8*3, 0, -8*2)
  self.shape:setFilterData(cat.wall, cat.player, 0)
  self.shape:setSensor(true)
  self.unlocked = (ident == 1)
end

function class:destroy()
  self.ident = nil
  self.shape = nil
  self.kind = nil
  parent.destroy(self)
end

function class:join(name)
  local lvl = self.lvl
  local x, y = self:getPosition()
  local t = 4 + 2
  if name == "p1" then
    t = -t
  end
  self.unlocked = true
  local obj = lvl.newObject("player", x + t, y, name)
  if self.kind == "left" then
    obj:setScale(-1, 1)
  end
  obj.checkpoint = self
  game.updateCamera(lvl, 0)
  return obj
end

function class:update(dt)
  local lvl = self.lvl
  local d = lvl.data
  local list = {}
  self:detect(game.cat.player, list)
  for i, user in ipairs(list) do
    if user.checkpoint.ident < self.ident then
      user.checkpoint = self
      self:playSound("checkpoint")
      self.unlocked = true
      if self.ident == #lvl.def.checkpoint then
        lvl.destroyObject(user)
        if d.controlled <= 1 then
          d.over = 3
          d.complete = true
          app.say("cleared")
        end
      else
        app.say("checkpoint", 2)
      end
    end
  end
  
  if self.kind ~= "hidden" then
    local i = 1
    if self.unlocked then
      local on = lvl.data.elapsed%2 >= 1
      i = on and 2 or 3
    end
    self:setGraphic("checkpoint", i)
  end
end

return class