local cat = game.cat

local parent = require("obj.trigger")
local class = game.oo.class(parent)

function class:init(lvl, x, y)
  parent.init(self, lvl, x, y, 4)
  self:setGraphic("collapsing", 1)
  local body = self.body
  body:setType("static")
  body:newEdge(-4, 4, 4, 4)
  body:newEdge(-4, -4, -4, 4)
  body:newEdge(4, -4, 4, 4)
  self.up = body:newEdge(-4, -4, 4, -4)
  self.up:setFriction(0.5)
  self.sensor = body:newBox(4, 4)
  self.sensor:setSensor(true)
  body:setFilter(cat.wall, cat.default, 0)
end

function class:setMode(on)
  local c = on and cat.wall or cat.none
  local fixtures = self.body:getFixtures()
  for i, fixture in ipairs(fixtures) do
    if not fixture:isSensor() then
      fixture:setFilter(c, cat.default, 0)
    end
  end
end

function class:update(dt)
  parent.update(self, dt)
  if self.wait then
    self.wait = self.wait - dt
    if self.wait <= 0 then
      if self.found then
        self:toggle(false)
      else
        self:toggle(true)
      end
    end
  end
end

function class:toggle(on)
  if on then
    if not self:detect("dynamic") then
      self:setGraphic("collapsing", 1)
      self.sprite:setVisible(true)
      self:setMode(true)
      self.wait = nil
    end
  else
    self:playSound("collapse")
    self:setGraphic("collapsing", 3)
    self:setMode(false)
    self.wait = 6
    self.found = false
    self:debree(8, 0, 0, 255, 255, 255)
  end
end

function class:collision(other, contact)
  if not self.found then
    local nx, ny = contact:getNormal()
    if ny < 0 then
      --local f1, f2 = contact:getFixtures()
      --local c1 = f1:getFilterData()
      --local c2 = f2:getFilterData()
      --if c1 == cat.player or c2 == cat.player then
      if other.type == "player" then
        self.found = true
        self.wait = 1
        self:setGraphic("collapsing", 2)
        self:playSound("activate")
      end
    end
  end
end

function class:shot(pts)
  --self:playSound("crush")
  self.found = true
  self.wait = 0
end

function class:hit(pts)
  if pts >= 5 then
    self.found = true
    self.wait = 0
  end
end

return class