local cat = game.cat
local lg = love.graphics

local parent = require("obj.trigger")
local class = game.oo.class(parent)

function class:init(lvl, x, y, bits)
  local w, h = lvl.getDimensions()
  local v = 0
  for i = 1, h do
    if not lvl.isWalkable(self.index + i*w) then
      break
    end
    v = v + 1
  end
  if lvl.map[self.index + (v + 1)*w] == "lava" then
    v = v + 1
  end
  local height = (v + 1)*8
  local x2, y2 = x, y + height
  
  parent.init(self, lvl, x, y, 2)
  
  self.min = y
  self.max = y2
  
  local body = self.body
  self.up = body:newEdge(-4, -4, 4, -4)
  body:setFriction(0.5)
  body:setFilter(game.cat.platform, game.cat.default, 0)
  body:setType("kinematic")
  body:setGravityScale(0)
  
  local img = game.image.columnb
  img:setWrap("repeat", "repeat")
  local iw, ih = img:getDimensions()
  local quad = lg.newQuad(0, 0, iw, ih, iw, ih)
  local sprite = self.sprite
  local sprite_draw = sprite.draw
  sprite.draw = function(sprite)
    lg.push()
    lg.setColor(1, 1, 1, 1)
    local x1, y1 = self:getPosition()
    x1 = math.floor(x1 + 0.5)
    y1 = math.floor(y1 + 0.5)
    local h = (y2 - y1)
    if h > 0 then
      quad:setViewport(0, 0, iw, h)
      lg.translate(-iw/2, -ih/2)
      lg.draw(img, quad, x1, y1)
      lg.draw(game.image.columna, x1, y1)
    end
    lg.pop()
  end
  self:loopSound("click")
end

function class:destroy()
  self:stopLoop()
  parent.destroy(self)
end

function class:update(dt)
  self.detected = self:detect(cat.player)
  local x, y = self:getPosition()
  local vx, vy = 0, 0
  if not self.detected and y > self.min then
    vy = -20
  elseif self.detected and y < self.max then
    vy = 20
  end
  self:updateSound(self.loop, vy > 0 and 1 or 0)
  local x2, y2 = x, y + vy*dt
  y2 = math.max(y2, self.min)
  y2 = math.min(y2, self.max)
  self:movePlatform(x2, y2, self.up)
end

return class