local animations =
{
  file = "conveyor",
  left = { start = 1, frames = 4, cycle = "loop", steps = 3 },
  center = { start = 5, frames = 4, cycle = "loop", steps = 3 },
  right = { start = 9, frames = 4, cycle = "loop", steps = 3 },
  both = { start = 13, frames = 4, cycle = "loop", steps = 3 },
}

local parent = require("obj.body")
local class = game.oo.class(parent)

class.animations = animations

function class:init(lvl, x, y, bits)
  parent.init(self, lvl, x, y, 5)
  local s = "center"
  if bits%8 >= 4 and bits%16 >= 8 then
    s = "both"
  elseif bits%8 >= 4 then
    s = "left"
  elseif bits%16 >= 8 then
    s = "right"
  end

  local body = self.body
  body:newEdge(-4, 4, 4, 4)
  if bits%8 >= 4 then
    body:newEdge(-4, -4, -4, 4)
  end
  if bits%16 >= 8 then
    body:newEdge(4, -4, 4, 4)
  end
  self.top = body:newEdge(-4, -4, 4, -4)
  body:setFilter(game.cat.wall, game.cat.default, 0)

  self.speed = -1
  self:setAnimation(self.animations[s])
end

function class:destroy()
  self.top = nil
  self.speed = nil
  self.animation = nil
  self.elapsed = nil
  parent.destroy(self)
end

function class:update(dt)
  --parent.update(self, dt)
  local top = self.top
  local contacts = self.body:getContacts()
  for i, c in ipairs(contacts) do
    if c:isTouching() and c:isEnabled() then
      local f1, f2 = c:getFixtures()
      if f1 == top or f2 == top then
        local nx, ny = c:getNormal()
        if ny < 0 then
          c:setFriction(1)
          c:setTangentSpeed(self.speed)
        else
          c:setFriction(0)
        end
      end
    end
  end
end

return class