local cat = game.cat

local states = {}

states.spawn = {}
function states.spawn:enter(obj)
  obj.elapsed = math.random()*3
  obj.body:setActive(true)
  obj.body:setGravityScale(0)
  obj.body:setLinearVelocity(0, 0)
  obj.shape:setSensor(true)
end
function states.spawn:update(obj, dt)
  if obj.elapsed >= 3 then
    obj.elapsed = obj.elapsed%3
    if obj:detect(cat.player) then
      obj:playSound("surprise")
      obj.body:setGravityScale(1)
      obj.shape:setSensor(false)
      obj.spawns = obj.spawns + 1
      obj.health = 3
      obj.invincible = 0
      obj:setState("raise")
    end
  end
end

states.raise = {}
function states.raise:update(obj, dt)
  if obj.elapsed >= obj.animation.total then
    obj.body:setActive(true)
    obj:setState("crawling")
  end
end

states.crawling = {}
function states.crawling:enter(obj)
  obj.limit = math.random(3, 9)
end
function states.crawling:exit(obj)
  obj.limit = nil
end
function states.crawling:update(obj, dt)
  if obj.elapsed >= obj.animation.total then
    local mx, my = obj:getMovement()
    local sx, sy = obj:getScale()
    if mx == -1 or mx == 1 then
      if sx ~= mx then
        obj.elapsed = 0
        obj:setScale(mx, 1)
      end
    end
  end
  local sx, sy = obj:getScale()
  local vx, vy = obj:getVelocity()
  obj:setVelocity(sx*obj.speed, vy)
  if obj.elapsed >= obj.limit then
    obj:setState("stomp")
  end
end
function states.crawling:fall(obj)
  obj:setState("falling")
end

states.falling = {}
function states.falling:update(obj, dt)
  if obj.grounded then
    if obj.health > 0 then
      obj:setState("crawling")
    else
      obj:setState("dead")
    end
  end
end

states.stomp = {}
function states.stomp:enter(obj)
  obj:setVelocity(0, 0)
  obj.body:setFilter(cat.enemy, cat.wall + cat.platform, -2)
end
function states.stomp:exit(obj)
  obj.body:setFilter(cat.enemy, cat.default, -2)
end
function states.stomp:update(obj, dt)
  --obj.body:setActive(false)
  if obj.elapsed >= 3 then
    obj:setState("raise")
  end
end
function states.stomp:fall(obj)
  obj:setState("falling")
end

states.dead = {}
function states.dead:enter(obj)
  game.screenshake(obj.lvl, 0.5)
  obj:debree(8, 0, 0, 255, 0, 0)
  obj:setGraphic()
  obj:setVelocity(0, 0)
  obj.body:setActive(false)
end
function states.dead:awake(obj)
  obj:setState("spawn")
end

return states