local cat = game.cat
local animations =
{
  file = "crawler",
  crawling = { start = 1, frames = 8, cycle = "loop", steps = 5 },
  falling = { start = 9, frames = 1, cycle = "none" },
  raise = { start = 10, frames = 5, cycle = "reverse", steps = 5 },
  stomp = { start = 10, frames = 5, cycle = "none", steps = 4 },
  spawn = { start = 14, frames = 1, cycle = "none" },
}

local parent = require("obj.agent")
local class = game.oo.class(parent)

class.states = require("obj.crawler-states")
class.animations = animations
class.bonus = 200

function class:init(lvl, x, y)
  parent.init(self, lvl, x, y)
  local body = self.body
  body:setLinearDamping(3)
  --body:setActive(false)
  self.shape = body:newBox(2, 3, 0, 0.5)
  self.shape:setFilterData(cat.enemy, cat.default, -2)
  self.shape:setFriction(1)
  self.sensor = body:newBox(8*16, 8*1, 0, -8*0.5)
  self.sensor:setFilterData(cat.enemy, cat.default, -2)
  self.sensor:setSensor(true)
  self.speed = math.random(15, 20)
  self.spawns = 0
  self:setState("spawn")
end

function class:destroy()
  self:setState()
  self.spawns = nil
  self.shape = nil
  self.speed = nil
  parent.destroy(self)
end

function class:getMovement()
  local sx, sy = self:getScale()
  local mx, my = sx, 0
  local front = self:isBlocked(0, 0, 0.5, 0)
  local below = self:isBlocked(0, 0, 0.25, 1)
  if front or not below then
    -- hurdles
    mx = -mx
  else
    local other = self:detect(cat.player)
    if other then
      local x1, y1 = self:getPosition()
      local x2, y2 = other:getPosition()
      if x1 < x2 then
        mx = 1
      else
        mx = -1
      end
    end
  end
  return mx, my, false, false
end

function class:onDamage(h)
  if h > 0 then
    self:playSound("shake")
  end
end

function class:onDie(pts)
  self:dropBonus()
  self:setState("falling")
end

function class:stomp(other, contact)
  if self.health > 0 and self.grounded then
    self:playSound("collapse")
    contact:setRestitution(1)
    self:setState("stomp")
    return false
  end
end

function class:collision(other, contact)
  if self.health > 0 then
    other:call("hit", 1)
  end
end

function class:shot(pts)
  self:damage(pts)
end

return class