local states = {}

states.spawn = {}
function states.spawn:update(obj, dt)
  obj.body:setActive(true)
  obj.spawns = obj.spawns + 1
  obj.health = 12
  obj.invincible = 0
  obj:setState("standing", 0)
end

states.standing = {}
function states.standing:enter(obj, wait)
  obj.wait = wait or math.random()*5 + 1
end
function states.standing:exit(obj)
  obj.wait = nil
end
function states.standing:update(obj, dt)
  if obj.elapsed >= obj.wait then
    if obj:detect(game.cat.player) then
      obj:setState("strike")
    end
  end
end

states.strike = {}
function states.strike:update(obj, dt)
  if obj.elapsed >= obj.animation.total then
    obj:playSound("crush")
    game.screenshake(obj.lvl, 1.5)
    obj:setState("attack")
  end
end

states.attack = {}
function states.attack:enter(obj)
  local lvl = obj.lvl
  local tx, ty = obj:getTile()
  local w, h = lvl.getDimensions()
  local list, exists = lvl.flood(obj.index + w)
  for i = #list, 1, -1 do
    local index = list[i]
    local c = lvl.cells[index]
    if c and c.bits%2 < 1 then
      table.remove(list, i)
    end
  end
  obj.queue = list
end
function states.attack:exit(obj)
  obj.queue = nil
end
function states.attack:update(obj, dt)
  local period = 0.5
  if obj.lvl.data.tick%10 == 0 then
    local range = 8*8
    local p = (1 - obj.elapsed/period)
    local x1, y1 = obj:getPosition()
    for i, w in ipairs(obj.queue) do
      local v = obj.lvl.cells[w]
      local x2, y2 = v:getPosition()
      local dx, dy = x2 - x1, y2 - y1
      local dist = math.sqrt(dx*dx + dy*dy)
      if dist <= range then
        local r = dist/range
        if math.random(10) >= 20*r*p then
          v:spawn("debrisb", 0, 4 + 2)
        end
      end
    end
  end
  if obj.elapsed >= period then
    obj:setState("standing")
  end
end

states.dead = {}
function states.dead:enter(obj)
  obj:playSound("shake")
  obj:dropBonus()
  obj:debree(20, 0, 0, 255, 0, 0)
  obj.body:setActive(false)
  obj:setGraphic()
  game.screenshake(obj.lvl, 0.25)
end
function states.dead:awake(obj)
  obj:setState("spawn")
end

return states