local states = {}

states.spawn = {}
function states.spawn:update(obj, dt)
  obj:reposition()
  obj.body:setActive(false)
  obj.spawns = obj.spawns + 1
  obj.health = 1
  obj.invincible = 0
  obj:setState("idle")
end

states.idle = {}
function states.idle:enter(obj)
  obj.wait = math.random(5, 20)/10
end
function states.idle:exit(obj)
  obj.wait = nil
end
function states.idle:update(obj, dt)
  if obj.elapsed >= obj.wait then
    obj:setState("windup")
  end
end

states.windup = {}
function states.windup:update(obj, dt)
  if obj.elapsed > obj.animation.total then
    obj:setState("jumping")
  end
end

states.jumping = {}
function states.jumping:enter(obj)
  obj.body:setActive(true)
  obj:setVelocity(0, -obj.speed)
  obj:playSound("jump2")
end
function states.jumping:update(obj, dt)
  obj:dustTrail(dt)
  local x, y = obj:getPosition()
  local vx, vy = obj:getVelocity()
  if y > obj.iy and vy > 0 then
    obj:setState("landing")
  end
end

states.landing = {}
function states.landing:enter(obj)
  obj:reposition()
  obj.body:setActive(false)
  obj:playSound("splash")
end
function states.landing:update(obj, dt)
  if obj.elapsed >= 0.5 then
    obj:setState("idle")
  end
end

states.explode = {}
function states.explode:update(obj, dt)
  obj.body:setLinearVelocity(0, 0)
  obj.body:setActive(false)
  if obj.elapsed >= obj.animation.total then
    obj:setState("dead")
  end
end

states.dead = {}
function states.dead:enter(obj)
  obj:setGraphic()
end
function states.dead:awake(obj)
  obj:setState("spawn")
end

return states