local parent = require("obj.agent")
local class = game.oo.class(parent)
local cat = game.cat

class.nosleep = true
class.radius = 2
class.speed = 70
class.range = 8*100
class.lifetime = class.range/class.speed
class.graphic = "eggs"

function class:init(lvl, x, y, nx, ny, mask)
  mask = mask or cat.player
  parent.init(self, lvl, x, y, 8)
  local body = self.body
  --body:setType("dynamic")
  body:setGravityScale(0)
  body:setLinearVelocity(self.speed*nx, self.speed*ny)
  self.shape = body:newCircle(self.radius)
  self.shape:setFilterData(cat.enemy, cat.wall + cat.bullet + mask, -2)
  self.shape:setSensor(true)
  self.elapsed = 0
  self.health = 1
  self.lockout = -1
  self:playSound("egg")
end

function class:destroy()
  self.elapsed = nil
  self.shape = nil
  parent.destroy(self)
end

function class:update(dt)
  parent.update(self, dt)
  local x, y = self:getPosition()
  self.sprite:setPosition(x, y)
  if self.elapsed >= 0.1 then
    self.shape:setSensor(false)
  end
  if self.lvl.data.tick%3 == 0 then
    local n = "trail"..math.random(1, 3)
    local anim = self:spawn("animation", 0, 0, 7, n)
    anim:setColor(128, 128, 128)
  end
  if self.elapsed >= self.lifetime then
    self.lvl.destroyObject(self)
  end
end

function class:onDie(pts)
  self:spawn("explosion", 0, 0, 2, self.radius*6, 0.2)
  self:playSound("pop")
  self.lvl.destroyObject(self)
end

function class:shot(pts)
  self:damage(pts)
end

function class:collision(other, contact)
  contact:setEnabled(false)
  if self.health > 0 then
    local f1, f2 = contact:getFixtures()
    local c1 = f1:getFilterData()
    local c2 = f2:getFilterData()
    if c1 == cat.wall or c2 == cat.wall then
      self.lockout = -1
      self:damage()
    end
    other:call("shot", 3)
    if other:getType() == "player" and math.random(3) == 1 then
      app.say("shellshock", 6)
    end
  end
end

--[[
function class:setAwake(on)
  if not on then
    self.lvl.destroyObject(self)
  end
end
]]

function class:sync()
  parent.sync(self)
  local vx, vy = self:getVelocity()
  local angle = math.atan2(vy, vx)
  local octant = math.floor(16*angle/(2*math.pi) + 16 + 0.5)%16
  self:setGraphic(self.graphic, octant + 1)
end

return class