local parent = require("obj.destructible")
local class = game.oo.class(parent)

function class:init(lvl, x, y, z)
  parent.init(self, lvl, x, y, z)
  self.elapsed = 0
end

function class:destroy()
  self.elapsed = nil
  parent.destroy(self)
end

function class:setAwake(on)
  if self.nosleep then
    return
  end
  self.sprite:setVisible(on)
  self.body:setAwake(on)
  if on then
    self:message("awake")
  else
    self:message("sleep")
  end
  if self.loop and not on then
    self.loop:setVolume(0)
  end
end

function class:setState(s, ...)
  assert(s == nil or self.states[s], "invalid state")
  local a = self.animations and self.animations[s]
  self:setAnimation(a)
  -- exit previous state
  self:message("exit")
  self.prevstate = self.state
  self.state = self.states and self.states[s]
  self.elapsed = 0
  -- enter next state
  self:message("enter", ...)
end

function class:update(dt)
  parent.update(self, dt)
  self.elapsed = self.elapsed + dt
  self:message("update", dt)
end

function class:message(msg, ...)
  local state = self.state
  if state then
    local func = state[msg]
    if func then
      return func(state, self, ...)
    end
  end
end

return class