local states = {}

states.spawn = {}
function states.spawn:update(obj, dt)
  obj.body:setActive(true)
  obj.spawns = obj.spawns + 1
  obj.health = 1
  obj.invincible = 0
  obj:setState("standing")
end

states.standing = {}
function states.standing:enter(obj, wait)
  obj.wait = wait or math.random(5, 100)/10
  obj:setVelocity(0, 0)
  obj.body:setFriction(1)
end
function states.standing:exit(obj)
  obj.body:setFriction(0)
  obj.wait = nil
end
function states.standing:update(obj, dt)
  local mx, my, jump, shoot = obj:getMovement()
  if mx ~= 0 then
    if obj.elapsed >= obj.wait then
      local sx = math.random(2) == 1 and -1 or 1
      obj:setScale(sx, 1)
      obj:setState("walking")
    end
  else
    if shoot and obj.elapsed >= obj.cooldown then
      obj:setState("shooting")
    end
  end
end
function states.standing:fall(obj)
  obj:setState("falling")
end

states.walking = {}
function states.walking:update(obj, dt)
  local mx, my, jump, shoot = obj:getMovement()
  if mx == -1 or mx == 1 then
    obj:setScale(mx, 1)
  end
  local sx, sy = obj:getScale()
  local vx, vy = obj:getVelocity()
  obj:setVelocity(sx*obj.speed, vy)
  if mx == 0 or obj.elapsed >= 5 then
    obj:setState("standing")
  end
  if jump then
    obj:setState("jumping")
  end
  -- line of sight check
  if shoot and obj.elapsed >= 0.25 then
    obj:setState("shooting")
  end
end
function states.walking:fall(obj)
  obj:setState("falling")
end

states.shooting = {}
function states.shooting:enter(obj)
  local sx, sy = obj.sprite:getScale()
  obj:spawn("animation", 4*sx, -2, 8, "chargeup")
  obj:playSound("chargeup")
end
function states.shooting:update(obj, dt)
  obj:setVelocity(0, 0)
  if obj.elapsed >= 0.4 then
    local sx, sy = obj.sprite:getScale()
    obj:spawn(obj.weapon, 4*sx, -2, sx, 0, game.cat.player)
    obj:setState("standing", obj.cooldown*2)
  end
end
function states.shooting:fall(obj)
  obj:setState("falling")
end

states.jumping = {}
function states.jumping:enter(obj)
  local xv, yv = obj:getVelocity()
  obj:setVelocity(xv, -160)
  obj:playSound("jump2")
end
function states.jumping:update(obj, dt)
  local sx, sy = obj:getScale()
  local xv, yv = obj:getVelocity()
  obj:setVelocity(sx*obj.speed, yv)
  if yv > 0 then
    obj:setState("falling")
  end
end
function states.jumping:hitroof(obj)
  obj:setState("falling")
end
function states.jumping:hitground(obj, yv)
  obj:setState("walking")
end

states.falling = {}
function states.falling:update(obj, dt)
  local sx, sy = obj:getScale()
  local vx, vy = obj:getVelocity()
  obj:setVelocity(sx*obj.speed, vy)
  if obj.grounded then
    if obj.health > 0 then
      obj:setState("landing")
    else
      obj:setState("dead")
    end
  end
end

states.landing = {}
function states.landing:enter(obj)
  obj:setVelocity(0, 0)
end
function states.landing:update(obj, dt)
  if obj.elapsed > 0.25 then
    obj:setState("standing", 0.5)
  end
end
function states.landing:fall(obj)
  obj:setState("falling")
end

states.dead = {}
function states.dead:enter(obj)
  obj:debree(8, 0, 0, 255, 0, 0)
  obj:setVelocity(0, 0)
  obj.body:setActive(false)
end
function states.dead:awake(obj)
  obj:setState("spawn")
end

return states