local animations =
{
  file = "gunman",
  standing = { start = 1, frames = 1, cycle = "none" },
  shooting = { start = 1, frames = 1, cycle = "none" },
  walking = { start = 2, frames = 6, cycle = "loop", steps = 3 },
  jumping = { start = 8, frames = 1, cycle = "none" },
  falling = { start = 9, frames = 1, cycle = "none" },
  landing = { start = 10, frames = 6, cycle = "none" },
  dead = { start = 16, frames = 1, cycle = "none" },
  spawn = { start = 16, frames = 1, cycle = "none" },
}

local parent = require("obj.agent")
local class = game.oo.class(parent)

class.states = require("obj.gunman-states")
class.animations = animations
class.bonus = 400
class.cooldown = 0.4
class.weapon = "bullet"
class.range = 8*20

function class:init(lvl, x, y)
  parent.init(self, lvl, x, y)
  local body = self.body
  body:setLinearDamping(3)
  self.shape = body:newBox(2, 3, 0, 0.5)
  self.shape:setFilterData(game.cat.enemy, game.cat.default, -2)
  self.speed = math.random(30, 35)
  self:setState("spawn")
end

function class:destroy()
  self:setState()
  self.shape = nil
  self.speed = nil
  parent.destroy(self)
end

function class:getType()
  return "gunman"
end

function class:getMovement()
  local sx, sy = self:getScale()
  local mx, my = sx, 0
  local jump, shoot = false, false
  local front = self:isBlocked(0, 0, 0.5, 0)
  local below = self:isBlocked(0, 0, 0, 1)
  local lojump = self:canJump(1, -1)
  local hijump = self:canJump(1, -2)
  local canjump = lojump or hijump
  if front then
    -- hurdles
    if canjump or math.random(5) == 1 then
      jump = true
    else
      mx = -mx
    end
  end
  if not below and canjump then
    -- holes
    jump = true
  end
  if not lojump and hijump and math.random(5) == 1 then
    -- platforms
    jump = true
  end
  if not jump then
    if self:lineOfSight(16, 0) then
      shoot = true
    end
  end
  return mx, my, jump, shoot
end

function class:onDamage(h)
  if h == 0 then
    self:playSound("shake")
    game.screenshake(self.lvl, 0.25)
  end
end

function class:onDie(pts)
  self:dropBonus()
  self:setState("falling")
end

function class:collision(other, contact)
  if self.health > 0 then
    other:call("hit", 1)
  end
end

function class:hit(pts)
  if pts >= 10 then
    self:damage()
  end
end

function class:shot(pts)
  self:damage(pts)
end

return class