local cat = game.cat
local states = {}

states.spawn = {}
function states.spawn:update(obj, dt)
  obj:reposition()
  obj.spawns = obj.spawns + 1
  obj.health = 1
  obj.invincible = 0
  obj:setState("appear")
end

states.appear = {}
function states.appear:update(obj, dt)
  if obj.elapsed >= obj.animation.total then
    obj.jumps = 0
    obj.body:setActive(true)
    obj:setState("idle")
  end
end

states.disappear = {}
function states.disappear:enter(obj)
  obj.body:setActive(false)
  obj:reposition()
end
function states.disappear:update(obj, dt)
  if obj.elapsed >= 3 then
    obj:setState("appear")
  end
end

states.idle = {}
function states.idle:enter(obj)
  obj.wait = math.random(5, 20)/10
  obj:setVelocity(0, 0)
  obj.body:setFilter(cat.enemy, cat.wall + cat.platform, -2)
end
function states.idle:exit(obj)
  obj.wait = nil
  obj.body:setFilter(cat.enemy, cat.default, -2)
end
function states.idle:update(obj, dt)
  if obj.elapsed >= obj.wait then
    obj:setState("windup")
  end
end
function states.idle:fall(obj)
  obj:setState("falling")
end

states.windup = {}
function states.windup:enter(obj)
  local nx = (math.random(2) == 1) and -1 or 1
  if obj:isPushedOnAxis(1, 0) then
    nx = -1
  elseif obj:isPushedOnAxis(-1, 0) then
    nx = 1
  end
  obj:setScale(nx, 1)
end
function states.windup:update(obj, dt)
  if obj.elapsed > obj.animation.total then
    obj:setState("jumping")
  end
end
function states.windup:fall(obj)
  obj:setState("falling")
end

states.jumping = {}
function states.jumping:enter(obj)
  local sx, sy = obj:getScale()
  obj:setVelocity(sx*obj.speed, -obj.speed)
  obj:playSound("jump2")
  obj.jumps = obj.jumps + 1
  obj.body:setActive(true)
end
function states.jumping:update(obj, dt)
  if obj.elapsed >= 0.1 and obj.grounded then
    obj:setState("idle")
  end
end

states.falling = {}
function states.falling:hitground(obj)
  obj:setState("landing")
end

states.landing = {}
function states.landing:enter(obj)
  obj.body:setFriction(1)
end
function states.landing:exit(obj)
  obj.body:setFriction(0)
end
function states.landing:update(obj, dt)
  if obj.elapsed >= obj.animation.total then
    if obj.jumps < 5 then
      obj:setState("idle")
    else
      obj:setState("disappear")
    end
  end
end
function states.landing:fall(obj)
  obj:setState("falling")
end

states.exploding = {}
function states.exploding:update(obj, dt)
  --obj:spawn("explosion", 0, 0, 0, 8*3, 0.25)
  obj:setGraphic()
  obj:setState("dead")
end

states.dead = {}
function states.dead:enter(obj)
  obj.body:setActive(false)
end
function states.dead:awake(obj, dt)
  obj:setState("spawn")
end

return states