local cat = game.cat
local animations =
{
  file = "hopper", oy = -6,
  appear = { start = 12, frames = 5, cycle = "none" },
  disappear = { start = 12, frames = 5, cycle = "reverse" },
  idle = { start = 6, frames = 1, cycle = "none" },
  windup = { start = 6, frames = 6, cycle = "none" },
  windupb = { start = 1, frames = 6, cycle = "reverse" },
  falling = { start = 1, frames = 1, cycle = "none" },
  jumping = { start = 1, frames = 1, cycle = "none" },
  landing = { start = 1, frames = 6, cycle = "none" },
}

local parent = require("obj.agent")
local class = game.oo.class(parent)

class.states = require("obj.hopper-states")
class.animations = animations
class.bonus = 200
class.speed = 180

function class:init(lvl, x, y, d)
  parent.init(self, lvl, x, y, 8)
  local body = self.body
  body:setLinearDamping(3)
  self.shape = body:newBox(3, 2)
  self.shape:setFilterData(cat.enemy, cat.wall + cat.platform, -2)
  self.shape:setFriction(0)
  self:setState("spawn")
end

function class:destroy()
  self:setState()
  self.shape = nil
  self.speed = nil
  --self.turned = nil
  parent.destroy(self)
end

function class:onDie(pts)
  self:debree(8, 0, -4, 255, 0, 0)
  self:dropBonus()
  self:setState("exploding")
end

function class:onDamage(h)
  game.screenshake(self.lvl, 0.25)
end

function class:collision(other, contact)
  if self.health > 0 then
    local f1, f2 = contact:getFixtures()
    local c1 = f1:getFilterData()
    local c2 = f2:getFilterData()
    if c1 == cat.player or c2 == cat.player then
      self.lockout = -1
      self:damage()
    end
    other:call("damage", 1)
  end
end

function class:shot(pts)
  self:damage(pts)
end

return class