local cat = game.cat
local states = {}

states.spawn = {}
function states.spawn:update(obj, dt)
  obj:reposition()
  obj.spawns = obj.spawns + 1
  obj.health = 1
  obj.invincible = 0
  obj:setState("idle")
end

states.idle = {}
function states.idle:enter(obj)
  obj.body:setActive(true)
  obj.wait = math.random(10, 20)/10
  obj:setVelocity(0, 0)
  obj.body:setFilter(cat.enemy, cat.wall + cat.platform, -2)
end
function states.idle:exit(obj)
  obj.wait = nil
  obj.body:setFilter(cat.enemy, cat.default, -2)
end
function states.idle:update(obj, dt)
  if obj.elapsed >= obj.wait then
    obj:setState("windupb")
  end
end
function states.idle:fall(obj)
  obj:setState("falling")
end

states.windupb = {}
function states.windupb:update(obj, dt)
  if obj.elapsed > obj.animation.total then
    obj:setState("jumping")
  end
end
function states.windupb:fall(obj)
  obj:setState("falling")
end

states.jumping = {}
function states.jumping:enter(obj)
  local heights = {}
  local gx, gy = obj.lvl.world:getGravity()
  for i = 1, 2 do
    local h = 8*i + 4
    local v = math.sqrt(2*gy*h)
    table.insert(heights, v)
  end
  local s = table.random(heights)
  obj:setVelocity(0, -s)
  obj:playSound("jump2")
  obj.body:setActive(true)
end
function states.jumping:update(obj, dt)
  if obj.elapsed >= 0.1 and obj.grounded then
    obj:setState("idle")
  end
  local vx, vy = obj:getVelocity()
  if vy > 0 then
    obj:spawn("bulletb", 0, 0, 1, 0)
    obj:spawn("bulletb", 0, 0, 1, 1)
    obj:spawn("bulletb", 0, 0, 0, 1)
    obj:spawn("bulletb", 0, 0, -1, 1)
    obj:spawn("bulletb", 0, 0, -1, 0)
    obj:setState("falling")
  end
end

states.falling = {}
function states.falling:hitground(obj)
  obj:setState("landing")
end

states.landing = {}
function states.landing:enter(obj)
  obj.body:setFriction(1)
end
function states.landing:exit(obj)
  obj.body:setFriction(0)
end
function states.landing:update(obj, dt)
  if obj.elapsed >= obj.animation.total then
    obj:setState("idle")
  end
end
function states.landing:fall(obj)
  obj:setState("falling")
end

states.exploding = {}
function states.exploding:update(obj, dt)
  obj:debree(8, 0, 0, 255, 0, 0)
  --obj:spawn("explosion", 0, 0, 0, 8*3, 0.25)
  obj:setGraphic()
  obj:setState("dead")
end

states.dead = {}
function states.dead:enter(obj)
  obj.body:setActive(false)
end
function states.dead:awake(obj, dt)
  obj:setState("spawn")
end

return states