local cat = game.cat
local states = {}

states.spawn = {}
function states.spawn:update(obj, dt)
  obj.sprite:setVisible(true)
  obj.body:setActive(true)
  obj.spawns = obj.spawns + 1
  obj.health = 1
  obj.invincible = 0
  obj:setVelocity(0, 0)
  obj:reposition()
  obj:setState("landing")
end

states.landing = {}
function states.landing:enter(obj)
  obj.wait = math.random(1, 3)
  obj:setVelocity(0, 0)
  obj:setScale(1, 1)
end
function states.landing:exit(obj)
  obj.wait = nil
end
function states.landing:update(obj, dt)
  if obj.elapsed >= obj.wait then
    obj:setState("jumping")
  end
end

states.hanging = {}
function states.hanging:enter(obj)
  obj.wait = math.random(1, 3)
  obj:setVelocity(0, 0)
  obj:setScale(1, -1)
end
function states.hanging:exit(obj)
  obj.wait = nil
end
function states.hanging:update(obj, dt)
  if obj.elapsed >= 3 then
    obj:setState("falling")
  end
end

states.jumping = {}
function states.jumping:enter(obj)
  obj.body:setType("dynamic")
  obj:playSound("jump2")
end
function states.jumping:exit(obj)
  obj.body:setType("kinematic")
end
function states.jumping:update(obj, dt)
  obj:setVelocity(0, -60)
  if obj:isPushedOnAxis(0, -1, cat.wall) then
    if obj.elapsed >= 0.1 then
      obj:spawn("bullet", -4, 0, -1, 0, cat.player)
      obj:spawn("bullet", 4, 0, 1, 0, cat.player)
    end
    obj:setState("hanging")
  end
end

states.falling = {}
function states.falling:enter(obj)
  obj.body:setType("dynamic")
end
function states.falling:exit(obj)
  obj.body:setType("kinematic")
end
function states.falling:update(obj, dt)
  obj:setVelocity(0, 60)
  if obj:isPushedOnAxis(0, 1, cat.wall) then
    if obj.elapsed >= 0.1 then
      obj:spawn("bullet", -4, 0, -1, 0, cat.player)
      obj:spawn("bullet", 4, 0, 1, 0, cat.player)
    end
    obj:setState("landing")
  end
end

states.exploding = {}
function states.exploding:update(obj, dt)
  obj:debree(8, 0, 0, 255, 0, 0)
  --obj:spawn("explosion", 0, 0, 0, 8*3, 0.25)
  obj:setState("dead")
end

states.dead = {}
function states.dead:enter(obj)
  obj.body:setActive(false)
  obj.sprite:setVisible(false)
end
function states.dead:awake(obj)
  obj:setState("spawn")
end

return states