local cat = game.cat
local animations =
{
  file = "jellyfish",
  landing = { start = 1, frames = 4, cycle = "none" },
  hanging = { start = 1, frames = 4, cycle = "none" },
  falling = { start = 4, frames = 3, cycle = "none" },
  jumping = { start = 4, frames = 3, cycle = "none" },
}

local parent = require("obj.agent")
local class = game.oo.class(parent)

class.states = require("obj.jellyfish-states")
class.animations = animations
class.bonus = 800

function class:init(lvl, x, y)
  parent.init(self, lvl, x, y)
  local body = self.body
  body:newCircle(4)
  body:setFilter(cat.enemy, cat.default, -2)
  body:setActive(false)
  body:setGravityScale(0)
  body:setType("kinematic")
  self:setState("spawn")
end

function class:destroy()
  self:setState()
  parent.destroy(self)
end

function class:onDamage(h)
  if h == 0 then
    self:playSound("shake")  
    game.screenshake(self.lvl, 0.25)
  end
end

function class:onDie(pts)
  self:dropBonus()
  self:setState("exploding")
end

function class:collision(other, contact)
  local f1, f2 = contact:getFixtures()
  local c1 = f1:getFilterData()
  local c2 = f2:getFilterData()
  if c1 ~= cat.wall and c2 ~= cat.wall then
    contact:setEnabled(false)
  end
  if self.health > 0 then
    other:call("hit", 10)
  end
end

function class:shot(pts)
  self:damage(pts)
end

return class