local cat = game.cat
local animations =
{
  file = "landmine",
  idle = { start = 1, frames = 1, cycle = "none" },
  blink = { start = 1, frames = 3, cycle = "loop", steps = 4 },
  exploding = { start = 4, frames = 1, cycle = "none" },
  dead = { start = 4, frames = 1, cycle = "none" },
}

local parent = require("obj.agent")
local class = game.oo.class(parent)

class.states = require("obj.landmine-states")
class.animations = animations

function class:init(lvl, x, y, bits)
  parent.init(self, lvl, x, y)
  local body = self.body
  body:setType("dynamic")
  body:setGravityScale(0)
  self.radius = 8*3.75
  self.shape = body:newBox(4, 8/4, 0, 4)
  self.shape:setFilterData(cat.enemy, cat.player, 0)
  self.sensor = body:newCircle(self.radius)
  self.sensor:setFilterData(cat.bullet, cat.default, -4)
  self.sensor:setSensor(true)
  self:setState("spawn")
end

function class:destroy()
  self:setState()
  parent.destroy(self)
end

function class:getNearby(cat)
  local x1, y1 = self:getPosition()
  local list = {}
  self:detect(cat, list)
  for i = #list, 1, -1 do
    local other = list[i]
    local x2, y2 = other:getPosition()
    local found = self:rayCast("wall", x1, y1, x2, y2)
    if found and found ~= other then
      table.remove(list, i)
    end
  end
  return list
end

function class:explode()
  self:spawn("explosion", 0, 0, 0, self.radius, 0.25)
  local list = self:getNearby()
  for i, obj in ipairs(list) do
    obj:call("damage", 10)
    if math.random(2) == 1 and obj:getType() == "player" then
      local n = table.random({ "kaboom", "blowedup", "bombdiggity" })
      app.say(n, 6)
    end
  end
  game.screenshake(self.lvl, 0.25)
  self:playSound("shake")
end

function class:collision(other, contact)
  self:message("explode")
end

return class