local parent = require("obj.agent")
local class = game.oo.class(parent)

local dirs =
{
  e = 0,
  w = math.pi,
  n = -math.pi/2,
  s = math.pi/2,
}
local normals =
{
  e = { 1, 0 },
  w = { -1, 0 },
  n = { 0, -1 },
  s = { 0, 1 },
}

class.nosleep = true

function class:init(lvl, x, y, h, w)
  parent.init(self, lvl, x, y, 7)
  self.body:setType("kinematic")
  self.weapon = w
  self.normal = normals[h]
  self.period = 2
  self.cycle = 0.5
  self.time = 0
  self.on = false
  self.sprite:setRotation(dirs[h or "s"])
  self:setGraphic("mount")
end

function class:destroy()
  self.weapon = nil
  self.normal = nil
  self.period = nil
  self.cycle = nil
  self.time = nil
  self.on = nil
  parent.destroy(self)
end

function class:setPeriod(p, d, t)
  self.period = p
  self.cycle = d
  self.time = t or 0
end

function class:update(dt)
  parent.update(self, dt)
  local lvl = self.lvl
  local e = self.time + lvl.data.elapsed
  local p = self.period
  local c = self.cycle
  local on = false
  if type(c) == "number" then
    on = (e + p/2)%p > p*c
  else
    local i = math.floor(e/p)%(#c)
    on = c[i + 1]
  end
  if on ~= self.on then
    self.on = on
    if on then
      local nx, ny = unpack(self.normal)
      self.inst = self:spawn(self.weapon, 0, 0, nx, ny)
    else
      self.inst:call("interrupt")
    end
  end
end

return class