local cat = game.cat
local states = {}

states.spawn = {}
function states.spawn:update(obj, dt)
  obj.spawns = obj.spawns + 1
  obj.health = 6
  obj.invincible = 0
  obj.body:setActive(true)
  obj.body:setLinearVelocity(0, 0)
  obj:setState("standing")
end

states.standing = {}
function states.standing:enter(obj)
  obj:setVelocity(0, 0)
end
function states.standing:update(obj, dt)
  if obj.elapsed >= 0.5 then
    obj.elapsed = obj.elapsed%0.5
    if obj:detect(cat.player) then
      if math.random(2) == 1 then
        obj:setState("crawling")
      else
        local sx, sy = obj:getScale()
        obj.body:setGravityScale(-sy)
        obj.body:setAwake(true)
        obj:setScale(sx, -sy)
        obj:setState("falling")
      end
    end
  end
end
function states.standing:fall(obj)
  obj:setState("falling")
end

states.crawling = {}
function states.crawling:enter(obj)
  obj.limit = math.random(1, 3)
end
function states.crawling:exit(obj)
  obj.limit = nil
end
function states.crawling:update(obj, dt)
  local mx, my = obj:getMovement()
  local sx, sy = obj:getScale()
  if mx == -1 or mx == 1 then
    sx = mx
    if sx ~= mx then
      obj.elapsed = 0
    end
  end
  obj:setScale(sx, sy)
  local vx, vy = obj:getVelocity()
  obj:setVelocity(sx*obj.speed, vy)
  if mx == 0 or obj.elapsed >= obj.limit then
    obj:setState("standing")
  end
end
function states.crawling:fall(obj)
  obj:setState("falling")
end

states.falling = {}
function states.falling:enter(obj)
  obj:playSound("jump2")
end
function states.falling:update(obj, dt)
  if obj.grounded then
    obj:setState("landing")
  end
end

states.landing = {}
function states.landing:enter(obj)
  local gs = obj.body:getGravityScale()
  local g = (gs > 0) and 1 or -1
  obj:spawn("bullet", -4, g*2, -1, 0, cat.player)
  obj:spawn("bullet", 4, g*2, 1, 0, cat.player)
end
function states.landing:update(obj, dt)
  if obj.elapsed >= 0.25 then
    obj:setState("crawling")
  end
end
function states.landing:fall(obj)
  obj:setState("falling")
end

states.dead = {}
function states.dead:enter(obj)
  obj:playSound("shake")
  obj:debree(16, 0, 0, 255, 0, 0)
  obj:dropBonus()
  obj:setGraphic()
  obj:setVelocity(0, 0)
  obj.body:setActive(false)
  game.screenshake(obj.lvl, 0.25)
end
function states.dead:awake(obj)
  obj:setState("spawn")
end

return states