local cat = game.cat
local animations =
{
  file = "octopus",
  standing = { start = 1, frames = 2, cycle = "loop", steps = 8 },
  crawling = { start = 2, frames = 8, cycle = "loop", steps = 2 },
  falling = { start = 10, frames = 1, cycle = "none" },
  landing = { start = 11, frames = 1, cycle = "none" },
}

local parent = require("obj.agent")
local class = game.oo.class(parent)

class.states = require("obj.octopus-states")
class.animations = animations
class.bonus = 1500
class.speed = 55

function class:init(lvl, x, y)
  parent.init(self, lvl, x, y)
  local body = self.body
  body:setMass(1000)
  self.shape = body:newBox(4, 7)
  self.shape:setFilterData(cat.enemy, cat.default, -2)
  self.shape:setFriction(1)
  self.sensor = body:newBox(8*18, 8*6)
  self.sensor:setFilterData(cat.enemy, cat.default, -2)
  self.sensor:setSensor(true)
  self:setState("spawn")
  self.dir = 0
end

function class:destroy()
  self:setState()
  self.dir = nil
  parent.destroy(self)
end

function class:onDie(pts)
  self:setState("dead")
end

function class:getMovement()
  local gs = self.body:getGravityScale()
  local g = (gs > 0) and 1 or -1
  local sx, sy = self:getScale()
  local mx, my = self.dir, 0
  if mx == 0 and self.lvl.data.tick%10 == 0 then
    local other = self:detect(cat.player)
    if other then
      local x1, y1 = self:getPosition()
      local x2, y2 = other:getPosition()
      if x1 < x2 then
        mx = 1
      else
        mx = -1
      end
      if math.random(3) == 1 then
        mx = -mx
      end
    end
  end
  if mx ~= 0 and mx == sx then
    local front = self:isBlocked(0, 0, 1.5, 0.5*g)
    local below = self:isBlocked(0, 0, 1.5, 2*g)
    if front or not below then
      mx = 0
    end
  end
  self.dir = mx
  return mx, my, false, false
end

function class:collision(other, contact)
  local f1, f2 = contact:getFixtures()
  local c1 = f1:getFilterData()
  local c2 = f2:getFilterData()
  if c1 ~= cat.wall and c2 ~= cat.wall then
    contact:setEnabled(false)
  end
  if self.health > 0 then
    other:call("hit", 10)
  end
end

function class:shot(pts)
  self:damage(pts)
end

return class