local states = {}

states.spawn = {}
function states.spawn:update(obj, dt)
  obj.body:setActive(true)
  obj.spawns = obj.spawns + 1
  obj.health = 4
  obj.invincible = 0
  obj:setState("standing")
end

states.standing = {}
function states.standing:enter(obj)
  obj.body:setFriction(1)
  obj.wait = math.random(5, 30)/10
end
function states.standing:exit(obj)
  obj.body:setFriction(0)
  obj.wait = nil
end
function states.standing:update(obj, dt)
  obj:setVelocity(0, 0)
  if obj.elapsed >= obj.wait then
    obj:setState("seeking")
  end
end
function states.standing:fall(obj)
  obj:setState("falling")
end
function states.standing:damage(obj, h)
  local sx, sy = obj:getScale()
  obj:setState("running", sx, false)
end
function states.standing:crash(obj, other, nx)
  obj:setState("bouncing", nx)
end

states.seeking = {}
function states.seeking:enter(obj)
  obj.body:setFriction(1)
  obj:setVelocity(0, 0)
end
function states.seeking:exit(obj)
  obj.body:setFriction(0)
end
function states.seeking:update(obj, dt)
  obj:setVelocity(0, 0)
  if obj.elapsed >= obj.animation.total then
    local x, y = obj:getPosition()
    local sx, sy = obj:getScale()
    if obj:lineOfSight(24*sx, 2) then
      obj:setState("surprise", 1)
    elseif obj:lineOfSight(-24*sx, 2) then
      obj:setState("surprise", -1)
    end
  end 
  if obj.elapsed >= obj.animation.total*4 then
    local sx, sy = obj:getScale()
    local jump = math.random(1, 2) == 1
    obj:setState("running", sx, jump)
  end
end
function states.seeking:fall(obj)
  obj:setState("falling")
end
function states.seeking:damage(obj, h)
  local sx, sy = obj:getScale()
  obj:setState("running", sx, false)
end

states.surprise = {}
function states.surprise:enter(obj, dir)
  obj:playSound("surprise")
  --app.playSound("surprise")
  obj:setScale(dir, 1)
  obj:setVelocity(0, -40)
end
function states.surprise:update(obj, dt)
  if obj.elapsed > 0.25 and obj.grounded then
    local sx, sy = obj:getScale()
    obj:setState("running", sx, false)
  end
end

states.running = {}
function states.running:enter(obj, dir, jump)
  obj.shouldjump = jump
  obj:setScale(dir, 1)
  obj:loopSound("step3")
end
function states.running:exit(obj)
  obj:stopLoop()
end
function states.running:update(obj, dt)
  obj:updateSound(obj.loop)
  local sx, sy = obj:getScale()
  local vx, vy = obj:getVelocity()
  obj:setVelocity(sx*obj.speed, vy)
  --if obj.elapsed%(1/60*3) - dt <= 0 then
  if obj.lvl.data.tick%3 == 0 then
    local n = "smoke"..math.random(1, 3)
    obj:spawn("animation", 0, 2, 1, n)
  end
  if obj.elapsed >= 3 or not obj:isBlocked(0, 0, 1.25, 2) then
    obj:setScale(-sx, sy)
    obj:setState("standing")
  elseif obj.shouldjump then
    if obj:canJump(-2, -1) and obj:isBlocked(0, 0, 2, 0) then
      obj:setVelocity(sx*obj.speed, -200)
      obj:setState("jumping")
    end
  end
end
function states.running:fall(obj)
  obj:setState("falling")
end
function states.running:crash(obj, other, nx)
  other:call("hit", 3)
  obj:setState("bouncing", nx)
end

states.jumping = {}
function states.jumping:enter(obj)
  obj:playSound("jump2")
end
function states.jumping:update(obj, dt)
  local vx, vy = obj:getVelocity()
  local sx, sy = obj:getScale()
  obj:setVelocity(obj.speed*sx, vy)
  if obj.grounded and obj.elapsed > 0.1 then
    obj:setState("landing")
  end
end
function states.jumping:crash(obj, other, nx)
  obj:setState("bouncing", nx)
end

states.falling = {}
function states.falling:update(obj, dt)
  if obj.grounded then
    if obj.health > 0 then
      obj:setState("landing")
    else
      obj:setState("dead")
    end
  end
end

states.landing = {}
function states.landing:update(obj, dt)
  if obj.elapsed >= 0.5 then
    obj:setState("standing")
  end
end
function states.landing:fall(obj)
  obj:setState("falling")
end

states.bouncing = {}
function states.bouncing:enter(obj, nx)
  local vx, vy = obj:getVelocity()
  local dx = math.random(5, 50)*nx
  obj:setVelocity(dx, vy - 80)
  game.screenshake(obj.lvl, 0.25)
  obj:playSound("shake")
end
function states.bouncing:exit(obj)
  local sx, sy = obj:getScale()
  obj:setScale(-sx, sy)
end
function states.bouncing:update(obj, dt)
  if obj.grounded and obj.elapsed >= 0.1 then
    obj:setState("standing")
  end
end

states.dead = {}
function states.dead:enter(obj)
  obj:debree(16, 0, 0, 255, 0, 0)
  local list = {}
  obj:detect(nil, list)
  for i, other in ipairs(list) do
    other:call("damage", 1)
  end
  obj:setGraphic()
  obj.body:setActive(false)
  obj:spawn("explosion", 0, 0, 0, 8*3, 0.25)
  obj:reposition()
end
function states.dead:awake(obj)
  obj:setState("spawn")
end

return states