local cat = game.cat
local animations =
{
  file = "ram",
  standing = { start = 9, frames = 1, cycle = "none" },
  seeking = { start = 13, frames = 4, cycle = "bounce", steps = 6 },
  surprise = { start = 8, frames = 1, cycle = "none" },
  running = { start = 1, frames = 6, cycle = "loop", steps = 2 },
  bouncing = { start = 7, frames = 1, cycle = "none" },
  jumping = { start = 8, frames = 1, cycle = "none" },
  falling = { start = 8, frames = 1, cycle = "none" },
  landing = { start = 9, frames = 4, cycle = "none" },
}

local parent = require("obj.agent")
local class = game.oo.class(parent)

class.states = require("obj.ram-states")
class.animations = animations
class.bonus = 500

function class:init(lvl, x, y)
  parent.init(self, lvl, x, y)
  local body = self.body
  body:setLinearDamping(3)
  self.shape = body:newBox(3, 3.8, 0, 1.9)
  self.shape:setFilterData(cat.enemy, cat.default, -2)
  self.sensor = body:newCircle(8*3.75)
  self.sensor:setFilterData(cat.enemy, cat.default, -2)
  self.sensor:setSensor(true)
  self.speed = 100
  self:setState("spawn")
end

function class:destroy()
  self.shape = nil
  self.speed = nil
  parent.destroy(self)
end

function class:onDamage(h)
  self:message("damage", h)
  if h == 0 then
    self:playSound("shake")
    game.screenshake(self.lvl, 0.25)
  end
end

function class:onDie(pts)
  self:dropBonus()
  self:setState("falling")
end

function class:collision(other, contact)
  other:call("hit", 5)
  if other:getType() == "player" and math.random(3) == 1 then
    app.say("bull", 6)
  end
  local nx, ny = contact:getNormal()
  local f1, f2 = contact:getFixtures()
  if f1 == self.shape then
    f1, f2 = f2, f1
    nx = -nx
  end
  local c = f1:getFilterData()
  if c == cat.wall then
    if math.abs(nx) > math.abs(ny) then
      self:message("crash", other, nx)
    end
  end
end

function class:shot(pts)
  self:damage(pts)
end

return class