local states = {}

states.spawn = {}
function states.spawn:update(obj, dt)
  obj.body:setActive(true)
  obj.spawns = obj.spawns + 1
  obj.health = 16
  obj.invincible = 0
  obj:setState("walking")
end

states.standing = {}
function states.standing:update(obj, dt)
  if obj.elapsed >= obj.animation.total*30 then
    obj:setState("walking")
  end
end
function states.standing:damage(obj)
  obj:setState("running")
end
function states.standing:fall(obj)
  obj:setState("falling")
end

states.walking = {}
function states.walking:update(obj, dt)
  local sx, sy = obj:getScale()
  local mx, my = obj:getMovement()
  if mx ~= 0 then
    sx = mx
  end
  obj:setScale(sx, sy)
  local vx, vy = obj:getVelocity()
  obj:setVelocity(sx*15, vy)
  if obj.elapsed >= obj.animation.total*10 then
    obj:setState("standing")
  end
end
function states.walking:damage(obj)
  obj:setState("running")
end
function states.walking:fall(obj)
  obj:setState("falling")
end

states.running = {}
function states.running:enter(obj)
  app.playSound("surprise")
  obj:loopSound("step3")
end
function states.running:exit(obj)
  obj:stopLoop()
end
function states.running:update(obj, dt)
  obj:updateSound(obj.loop)
  local sx, sy = obj:getScale()
  local mx, my = obj:getMovement()
  if mx ~= 0 then
    sx = mx
  end
  obj:setScale(sx, sy)
  if obj.lvl.data.tick%3 == 0 then
    local n = "smoke"..math.random(1, 3)
    obj:spawn("animation", 0, 6, 1, n)
  end
  local vx, vy = obj:getVelocity()
  obj:setVelocity(sx*60, vy)
  if obj.elapsed >= obj.animation.total*30 then
    obj:setState("standing")
  end
end
function states.running:damage(obj)
  obj.elapsed = obj.elapsed%obj.animation.total
end
function states.running:fall(obj)
  obj:setState("falling")
end

states.falling = {}
function states.falling:update(obj, dt)
  if obj.grounded then
    if obj.health > 0 then
      obj:setState("standing")
    else
      obj:setState("dead")
    end
  end
end

states.dead = {}
function states.dead:enter(obj)
  obj:playSound("crush")
  game.screenshake(obj.lvl, 0.5)
  obj:debree(24, 0, 0, 255, 0, 0)
  obj:setGraphic()
  obj:setVelocity(0, 0)
  obj.body:setActive(false)
end
function states.dead:awake(obj)
  obj:setState("spawn")
end

return states