local cat = game.cat
local animations =
{
  file = "rhino", ox = -10, oy = -18,
  standing = { start = 1, frames = 2, cycle = "loop", steps = 12 },
  walking = { start = 1, frames = 6, cycle = "loop", steps = 6 },
  running = { start = 1, frames = 6, cycle = "loop", steps = 2 },
  falling = { start = 7, frames = 1, cycle = "none" },
}

local parent = require("obj.agent")
local class = game.oo.class(parent)

class.states = require("obj.rhino-states")
class.animations = animations
class.bonus = 800

function class:init(lvl, x, y)
  parent.init(self, lvl, x, y)
  local body = self.body
  body:setLinearDamping(3)
  --body:setActive(false)
  self.shape = body:newBox(6, 6, 0, 3)
  self.shape:setFilterData(cat.enemy, cat.default, -2)
  self.shape:setFriction(1)
  self.spawns = 0
  self:setState("spawn")
end

function class:destroy()
  self:setState()
  self.spawns = nil
  self.shape = nil
  self.speed = nil
  parent.destroy(self)
end

function class:onDamage(h)
  self:message("damage", h)
  if h > 0 then
    self:playSound("shake")
  end
end

function class:onDie(pts)
  self:dropBonus()
  self:setState("falling")
end

function class:collision(other, contact)
  if self.health > 0 then
    other:call("hit", 1)
  end
end

function class:shot(pts)
  self:damage(pts)
end

function class:getMovement()
  local sx, sy = self:getScale()
  local mx, my = sx, 0
  local front = self:isBlocked(0, 0, 1, 0)
  local below = self:isBlocked(0, 0, 1, 2)
  if front or not below then
    -- hurdles
    mx = -mx
  end
  return mx, my, false, false
end

return class