local states = {}

states.spawn = {}
function states.spawn:update(obj, dt)
  obj.body:setActive(true)
  obj.spawns = obj.spawns + 1
  obj.health = 1
  obj.invincible = 0
  obj:setState("walking")
end

states.standing = {}
function states.standing:enter(obj)
  obj.wait = math.random(50, 250)/10
  obj:setVelocity(0, 0)
end
function states.standing:exit(obj)
  obj.wait = nil
end
function states.standing:update(obj, dt)
  if obj.elapsed >= obj.wait then
    obj:setState("walking")
  end
end
function states.standing:fall(obj)
  obj:setState("falling")
end

states.walking = {}
function states.walking:enter(obj)
  obj.wait = math.random(100, 300)/10
  if math.random(2) == 1 then
    local sx, sy = obj:getScale()
    obj:setScale(-sx, sy)
  end
end
function states.walking:exit(obj)
  obj.wait = nil
end
function states.walking:update(obj, dt)
  local mx, my = obj:getMovement()
  if mx == -1 or mx == 1 then
    obj:setScale(mx, 1)
  end
  local sx, sy = obj:getScale()
  local vx, vy = obj:getVelocity()
  obj:setVelocity(sx*obj.speed, vy)
  if obj.elapsed >= obj.wait then
    obj:setState("standing")
  end
end
function states.walking:fall(obj)
  obj:setState("falling")
end

states.falling = {}
function states.falling:update(obj, dt)
  if obj.grounded then
    if obj.health > 0 then
      obj:setState("walking")
    else
      obj:setState("dead")
    end
  end
end

states.dead = {}
function states.dead:enter(obj)
  obj:playSound("shake")
  game.screenshake(obj.lvl, 0.25)
  obj:debree(12, 0, 0, 255, 0, 0)
  obj:setVelocity(0, 0)
  obj.body:setActive(false)
  obj:setGraphic()
end
function states.dead:awake(obj)
  obj:setState("spawn")
end

return states