local animations =
{
  file = "shield",
  standing = { start = 6, frames = 2, cycle = "loop", steps = 12 },
  walking = { start = 1, frames = 5, cycle = "loop", steps = 6 },
  falling = { start = 8, frames = 1, cycle = "none" },
}
local cat = game.cat

local parent = require("obj.agent")
local class = game.oo.class(parent)

class.states = require("obj.shield-states")
class.animations = animations
class.bonus = 300

function class:init(lvl, x, y)
  parent.init(self, lvl, x, y)
  local body = self.body
  body:setLinearDamping(3)
  self.shape = body:newBox(2, 3, 0, 0.5)
  self.shape:setFilterData(cat.enemy, cat.default, -2)
  self.shape:setFriction(1)
  self.speed = math.random(15, 20)
  self.spawns = 0
  self:setState("spawn")
end

function class:destroy()
  self:setState()
  self.spawns = nil
  self.shape = nil
  self.speed = nil
  parent.destroy(self)
end

function class:getMovement()
  local sx, sy = self:getScale()
  local mx, my = sx, 0
  local front = self:isBlocked(0, 0, 0.5, 0)
  local below = self:isBlocked(0, 0, 0, 1)
  if front or not below then
    mx = -mx
  end
  return mx, my, false, false
end

function class:onDie(pts)
  self:dropBonus()
  self:setState("falling")
end

function class:shot(pts, other, contact)
  local sx, sy = self:getScale()
  local nx, ny = 0, 0
  if contact then
    nx, ny = contact:getNormal()
    local f1, f2 = contact:getFixtures()
    local b1, b2 = f1:getBody(), f2:getBody()
    if b2 == self.body then
      nx, ny = -nx, -ny
    end
  end
  if nx == sx then
    self:playSound("deflect")
    local vx, vy = other:getVelocity()
    local avx = math.abs(vx)
    other:setVelocity(nx*avx, vy)
    other.body:setFilter(cat.enemy, cat.wall + cat.player, -2)
    return false, true
  else
    self:playSound("shake")
    return self:damage(pts)
  end
end

function class:hit(pts)
  if self.health > 0 then
    self:playSound("shake")
    self:damage(pts)
  end
end

function class:collision(other, contact)
  if self.health > 0 then
    other:call("hit", 1)
  end
end

return class