local states = {}

states.spawn = {}
function states.spawn:enter(obj)
  obj.body:setActive(false)
  obj.body:setGravityScale(0)
  obj:setGraphic()
  obj.spawns = obj.spawns + 1
  obj.health = 30
  obj.invincible = 0
  obj.label = obj:spawn("label", 0, 2*8, 11*8, 3*8, app.lang.skullmaster1, "skullmaster1")
end
function states.spawn:update(obj, dt)
  if obj.sensor:detect(game.cat.player) then
    if obj.elapsed >= 3 then
      obj.lvl.data.master = obj
      obj:setState("appear")
    end
  else
    obj.elapsed = 0
  end
end

states.appear = {}
function states.appear:enter(obj)
  obj.body:setActive(true)
  obj:playSound("appear")
end
function states.appear:update(obj, dt)
  if obj.elapsed >= 0.75 then
    obj.lvl.destroyObject(obj.label)
    obj.label = nil
    obj:playSound("backflip")
    obj:setState("hovering")
  else
    if obj.lvl.data.tick%5 == 0 then
      local dx = math.random(-4, 4)
      local dy = math.random(-4, 4)
      local flash = obj:spawn("animation", dx, dy, 8, "spark1")
      flash:setColor(255, 0, 0)
    end
  end
end

states.hovering = {}
function states.hovering:enter(obj)
  obj.body:setGravityScale(0)
end
function states.hovering:exit(obj)
  obj.body:setAwake(true)
  obj.body:setGravityScale(1)
end
function states.hovering:update(obj, dt)
  obj:setVelocity(0, 0)
  if obj.elapsed > 3 then
    local target = obj.sensor:detect(game.cat.player)
    if target then
      local x1, y1 = obj:getPosition()
      local x2, y2 = target:getPosition()
      local dx, dy = x2 - x1, y2 - y1
      if math.abs(dx) < 8*4 and dy > 8*2 and dy < 8*6 then
        obj:setState("elbowing", x2, y2)
      else
        obj:setState("spewing")
      end
    end
  end
end

states.spewing = {}
function states.spewing:enter(obj)
  obj.body:setGravityScale(0)
  obj:spawn("animation", 0, -8, 8, "chargeup")
  obj:playSound("chargeup")
end
function states.spewing:exit(obj)
  obj.body:setAwake(true)
  obj.body:setGravityScale(1)
end
function states.spewing:update(obj, dt)
  obj:setVelocity(0, 0)
  if obj.elapsed >= 0.5 and obj.elapsed - dt < 0.5 then
    for i = 0, 6 do
      local r = i/6
      local x = math.cos(r*math.pi)
      local y = math.sin(r*math.pi)
      obj:spawn("egg", 0, -8, x, y)
    end
  elseif obj.elapsed > 1 then
    obj:setState("falling")
  end
end

states.elbowing = {}
function states.elbowing:enter(obj, tx, ty)
  --obj.body:setGravityScale(0)
  obj.target = { tx, ty }
end
function states.elbowing:exit(obj)
  --obj.body:setGravityScale(1)
  obj.target = nil
end
function states.elbowing:update(obj, dt)
  local x1, y1 = obj:getPosition()
  local x2, y2 = unpack(obj.target)
  local dx, dy = x2 - x1, y2 - y1
  local d = math.sqrt(dx*dx + dy*dy)
  if d > 200*dt then
    local nx, ny = dx/d, dy/d
    obj:setVelocity(nx*200, ny*200)
    obj:setScale(dx > 0 and 1 or -1, 1)
  else
    obj:setState("falling")
  end
end
function states.elbowing:hitground(obj, yv)
  obj:setState("landing")
end


states.standing = {}
function states.standing:enter(obj, wait)
  obj.wait = wait or math.random(5, 100)/10
  obj:setVelocity(0, 0)
  obj.body:setFriction(1)
end
function states.standing:exit(obj)
  obj.body:setFriction(0)
  obj.wait = nil
end
function states.standing:update(obj, dt)
  if obj.elapsed >= 0.5 then
    if math.random(2) == 1 then
      obj:setState("running")
    else
      obj:setState("jumping")
    end
  end
end
function states.standing:fall(obj)
  obj:setState("falling")
end

states.running = {}
function states.running:enter(obj)
  obj:loopSound("step3")
end
function states.running:exit(obj)
  obj:stopLoop()
end
function states.running:update(obj, dt)
  local sx, sy = obj:getScale()
  obj:setVelocity(sx*180, 0)
  local front = obj:isBlocked(0, 0, 1, 0)
  if front then
    obj:setScale(-sx, sy)
  end
  if obj.elapsed >= 3 then
    obj:setState("shooting")
  end
end
function states.running:fall(obj)
  obj:setState("falling")
end

states.shooting = {}
function states.shooting:enter(obj)
  app.playSound("surprise")
  --obj:spawn("animation", 4*sx, -2, 8, "chargeup")
end
function states.shooting:exit(obj)
  if obj.beam then
    obj.beam:interrupt()
    obj.beam = nil
  end
end
function states.shooting:update(obj, dt)
  obj:setVelocity(0, 0)
  if obj.elapsed >= 0.2 and obj.elapsed - dt < 0.2 then
    local sx, sy = obj.sprite:getScale()
    obj.beam = obj:spawn("beam", 12*sx, 0, sx, 0, game.cat.player)
    obj.beam.maxwidth = 4
    obj:playSound("chargeup")
  elseif obj.elapsed >= 0.8 then
    obj:setState("standing")
  end
end
function states.shooting:fall(obj)
  obj:setState("falling")
end

states.jumping = {}
function states.jumping:enter(obj)
  obj:setVelocity(0, -250)
  obj:playSound("jump2")
end
function states.jumping:update(obj, dt)
  local xv, yv = obj:getVelocity()
  if yv > 0 then
    obj:setState("hovering")
  end
end
function states.jumping:hitroof(obj)
  obj:setState("falling")
end
function states.jumping:hitground(obj, yv)
  obj:setState("standing")
end

states.falling = {}
function states.falling:enter(obj, nextup)
  obj.body:setGravityScale(1)
  obj.body:setAwake(true)
  obj.nextup = nextup or "landing"
end
function states.falling:update(obj, dt)
  if obj.grounded then
    if obj.health > 0 then
      obj:setState(obj.nextup)
    else
      obj:setState("dead")
    end
  end
end

states.confused = {}
function states.confused:update(obj, dt)
  obj:setVelocity(0, 0)
end

states.landing = {}
function states.landing:enter(obj)
  obj:setVelocity(0, 0)
end
function states.landing:update(obj, dt)
  if obj.elapsed > 0.25 then
    obj:setState("standing", 0.5)
  end
end
function states.landing:fall(obj)
  obj:setState("falling")
end

states.dead = {}
function states.dead:enter(obj)
  obj:playSound("shake")
  game.screenshake(obj.lvl, 1)
  obj:setVelocity(0, 0)
  obj.body:setActive(false)
  obj.lvl.call("stage", 1)
end
function states.dead:update(obj, dt)
  if obj.elapsed >= 3 then
    obj:dropBonus()
    obj.lvl.destroyObject(obj)
  else
    if obj.lvl.data.tick%5 == 0 then
      local dx = math.random(-4, 4)
      local dy = math.random(-4, 4)
      local flash = obj:spawn("animation", dx, dy, 8, "spark1")
      flash:setColor(255, 0, 0)
    end
    if obj.lvl.data.tick%20 == 0 then
      obj:playSound("shake")
    end
  end
end

return states