local cat = game.cat

local states = {}

states.spawn = {}
function states.spawn:enter(obj)
  obj.body:setActive(false)
  obj.body:setGravityScale(0)
  obj:setGraphic()
  obj.spawns = obj.spawns + 1
  obj.health = 60
  obj.invincible = 0
  obj.label = obj:spawn("label", 0, 3.5*8, 12*8, 5*8, app.lang.skullmaster2, "skullmaster2")
end
function states.spawn:update(obj, dt)
  if obj.sensor:detect(cat.player) then
    if obj.elapsed >= 3 then
      obj.lvl.data.master = obj
      obj:setState("appear")
    end
  else
    obj.elapsed = 0
  end
end

states.appear = {}
function states.appear:enter(obj)
  obj.body:setActive(true)
  obj:playSound("appear")
end
function states.appear:update(obj, dt)
  if obj.elapsed >= 0.75 then
    obj.lvl.destroyObject(obj.label)
    obj.label = nil
    obj:playSound("backflip")
    obj:setState("hovering")
  else
    if obj.lvl.data.tick%5 == 0 then
      local dx = math.random(-4, 4)
      local dy = math.random(-4, 4)
      local flash = obj:spawn("animation", dx, dy, 10, "spark1")
      flash:setColor(255, 0, 0)
    end
  end
end

states.hovering = {}
function states.hovering:enter(obj)
  obj.body:setGravityScale(0)
end
function states.hovering:exit(obj)
  obj.body:setAwake(true)
  obj.body:setGravityScale(1)
end
function states.hovering:update(obj, dt)
  obj:setVelocity(0, 0)
  if obj.elapsed > 3 or obj.health < 60 then
    local target = obj.sensor:detect(cat.player)
    if target then
      obj:setState("falling")
    end
  end
end

states.moving = {}
function states.moving:enter(obj)
  obj:loopSound("step3")
end
function states.moving:exit(obj)
  obj:stopLoop()
end
function states.moving:update(obj, dt)
  obj:updateSound(obj.loop)
  local sx, sy = obj:getScale()
  local vx, vy = obj:getVelocity()
  obj:setVelocity(sx*250, vy)
  if obj.lvl.data.tick%2 == 0 then
    local n = "smoke"..math.random(1, 3)
    obj:spawn("animation", 0, 8, 1, n)
  end
  if obj:isPushedOnAxis(sx, 0, cat.wall) then
    obj:setScale(-sx, 1)
    obj:setState("jumping")
  end
end
function states.moving:fall(obj)
  obj:setState("falling")
end

states.standing = {}
function states.standing:enter(obj, wait)
  obj.wait = wait or math.random(5, 15)/10
  obj:setVelocity(0, 0)
  obj.body:setFriction(1)
end
function states.standing:exit(obj)
  obj.body:setFriction(0)
  obj.wait = nil
end
function states.standing:update(obj, dt)
  if obj.elapsed >= obj.wait then
    if obj.sensor:detect(game.cat.player) then
      if math.random(2) == 1 then
        obj:setState("moving")
      else
        obj:setState("attacking")
      end
    else
      obj.elapsed = 0
    end
  end
end
function states.standing:fall(obj)
  obj:setState("falling")
end

states.attacking = {}
function states.attacking:enter(obj)
  app.playSound("surprise")
  obj:turn()
  local sx, sy = obj:getScale()
  obj.beam = obj:spawn("beam", 6*sx, 2, sx, 0)
  obj.beam.maxwidth = 8*1.25
end
function states.attacking:exit(obj)
  obj.beam:interrupt()
  obj.beam = nil
end
function states.attacking:update(obj, dt)
  obj:setVelocity(0, 0)
  if obj.elapsed >= 1 then
    obj:setState("standing")
  end
end
function states.attacking:fall(obj)
  obj:setState("falling")
end

states.jumping = {}
function states.jumping:enter(obj)
  obj:setVelocity(0, -250)
  obj:playSound("jump2")
end
function states.jumping:update(obj, dt)
  local vx, vy = obj:getVelocity()
  local sx, sy = obj:getScale()
  obj:setVelocity(sx*55, vy)
  if vy > 0 then
    obj:setState("falling")
  end
  --if obj.grounded then
    --obj:setState("landing")
  --end
end

states.falling = {}
function states.falling:enter(obj)
  obj.body:setAwake(true)
end
function states.falling:update(obj, dt)
  if obj.grounded then
    if obj.health > 0 then
      obj:setState("landing")
    else
      obj:setState("dead")
    end
  end
end

states.landing = {}
function states.landing:enter(obj)
  local lvl = obj.lvl
  obj:setVelocity(0, 0)
  local wall = obj:detect(cat.wall)
  local i = wall.index
  if lvl.map[i] == "wallb" then
    obj:playSound("crush")
    game.screenshake(lvl, 1.5)
    local list, exists = lvl.flood(i)
    for i = #list, 1, -1 do
      local index = list[i]
      local c = lvl.cells[index]
      if c and c.bits%2 < 1 then
        table.remove(list, i)
      end
    end
    obj.queue = list
  end
end
function states.landing:exit(obj)
  obj.queue = nil
end
function states.landing:update(obj, dt)
  local period = 0.5
  if obj.queue and obj.lvl.data.tick%10 == 0 then
    local range = 8*8
    local p = (1 - obj.elapsed/period)
    local x1, y1 = obj:getPosition()
    for i, w in ipairs(obj.queue) do
      local v = obj.lvl.cells[w]
      local x2, y2 = v:getPosition()
      local dx, dy = x2 - x1, y2 - y1
      local dist = math.sqrt(dx*dx + dy*dy)
      if dist <= range then
        local r = dist/range
        if math.random(10) >= 20*r*p then
          v:spawn("debrisb", 0, 6)
        end
      end
    end
  end
  if obj.elapsed > period then
    if obj.queue then
      obj:setState("jumping")
    else
      obj:setState("attacking")
    end
  end
end
function states.landing:fall(obj)
  obj:setState("falling")
end

states.dead = {}
function states.dead:enter(obj)
  obj:playSound("shake")
  game.screenshake(obj.lvl, 1)
  obj:setVelocity(0, 0)
  obj.body:setActive(false)
  obj.lvl.call("stage", 2)
end
function states.dead:update(obj, dt)
  if obj.elapsed >= 3 then
    obj:dropBonus()
    obj.lvl.destroyObject(obj)
  else
    if obj.lvl.data.tick%5 == 0 then
      local dx = math.random(-4, 4)
      local dy = math.random(-4, 4)
      local flash = obj:spawn("animation", dx, dy, 10, "spark1")
      flash:setColor(255, 0, 0)
    end
    if obj.lvl.data.tick%20 == 0 then
      obj:playSound("shake")
    end
  end
end

return states