local cat = game.cat

local states = {}

states.spawn = {}
function states.spawn:enter(obj)
  obj.body:setActive(false)
  obj.body:setGravityScale(0)
  obj:setGraphic()
  obj.spawns = obj.spawns + 1
  obj.health = 150
  obj.invincible = 0
  obj.label = obj:spawn("label", 0, 4.5*8, 9*8, 8*8, app.lang.skullmaster3, "skullmaster3")
end
function states.spawn:update(obj, dt)
  if obj.sensor:detect(cat.player) then
    if obj.elapsed >= 3 then
      obj.lvl.data.master = obj
      obj:setState("appear")
    end
  else
    obj.elapsed = 0
  end
end

states.appear = {}
function states.appear:enter(obj)
  obj.body:setActive(false)
  obj:setGraphic()
  obj:playSound("appear")
end
function states.appear:exit(obj)
  obj.body:setActive(true)
end
function states.appear:update(obj, dt)
  if obj.elapsed >= 2 then
    if obj.label then
      obj.lvl.destroyObject(obj.label)
      obj.label = nil
    end
    obj:playSound("backflip")
    obj:setState("hovering", 0)
  else
    if obj.lvl.data.tick%4 == 0 then
      local dx = math.random(-8, 8)
      local dy = math.random(-8, 8)
      local flash = obj:spawn("animation", dx, dy, 10, "spark1")
      flash:setColor(255, 0, 0)
    end
  end
end

states.hovering = {}
function states.hovering:enter(obj, wait)
  obj.wait = wait
end
function states.hovering:exit(obj)
  obj.wait = nil
end
function states.hovering:update(obj, dt)
  if obj.elapsed >= obj.wait then
    local other = obj.sensor:detect(cat.player)
    if other then
      local x1, y1 = obj:getPosition()
      local x2, y2 = other:getPosition()
      if y2 < y1 + 8*2 then
        if math.random(2) == 1 then
          obj:setState("attacking")
          return
        end
      end
    end
    local r = math.random(3)
    if r == 1 then
      obj:setState("spinning", "alternate")
    elseif r == 2 then
      obj:setState("spinning", "rotate")
    else
      obj:setState("spinning", "target")
    end
  end
end

states.spinning = {}
function states.spinning:enter(obj, attack)
  local other = obj.sensor:detect(cat.player)
  if not other then
    obj:setState("hovering", 2)
    return
  end
  obj.attack = attack
  obj.tick = 0
  obj.round = 0
  if attack == "target" then
    local x1, y1 = obj:getPosition()
    local x2, y2 = other:getPosition()
    local dx, dy = x2 - x1, y2 - y1
    local d = math.sqrt(dx*dx + dy*dy)
    local nx, ny = dx/d, dy/d
    obj.nx, obj.ny = nx, ny
  end
end
function states.spinning:exit(obj)
  obj.attack = nil
  obj.tick = nil
  obj.round = nil
  obj.nx, obj.ny = nil, nil
end
function states.spinning:update(obj, dt)
  local tick = obj.tick
  if obj.attack == "alternate" then
    if tick%40 == 0 then
      local n = 6
      if tick%80 == 0 then
        n = 7
      end
      for i = 0, n do
        local r = i/n*math.pi
        local nx = math.cos(r)
        local ny = math.sin(r)
        obj:spawn("egg", 0, 0, nx, ny)
      end
      obj.round = obj.round + 1
    end
  elseif obj.attack == "rotate" then
    if tick%10 == 0 then
      local n = 8
      local i = tick/10%(n + 1)
      local r = i*(1/n)*math.pi
      local nx = math.cos(r)
      local ny = math.sin(r)
      obj:spawn("egg", 0, 0, nx, ny)
      if i == n then
        obj.round = obj.round + 1
      end
    end
  elseif obj.attack == "target" then
    if tick%20 == 0 then
      obj:spawn("bigegg", 0, 0, obj.nx, obj.ny)
      obj.round = obj.round + 1
    end
  end
  obj.tick = tick + 1
  if obj.round == 3 then
    obj:setState("hovering", 2)
  end
end

states.attacking = {}
function states.attacking:enter(obj)
  app.playSound("surprise")
end
function states.attacking:exit(obj)
  if obj.beam then
    obj.beam:interrupt()
    obj.beam = nil
  end
end
function states.attacking:update(obj, dt)
  if obj.elapsed >= 0.4 and obj.elapsed - dt < 0.4 then
    local sx, sy = obj:getScale()
    obj.beam = obj:spawn("beam", 6*sx, 10, sx, 0)
    obj.beam.maxwidth = 8*1.25
  end
  if obj.elapsed >= 0.8 then
    obj:setState("hovering", 1)
  end
end

states.dead = {}
function states.dead:enter(obj)
  obj:playSound("shake")
  game.screenshake(obj.lvl, 1)
  obj:setVelocity(0, 0)
  obj.body:setActive(false)
  obj.lvl.call("stage", 3)
end
function states.dead:update(obj, dt)
  if obj.elapsed >= 3 then
    obj:dropBonus()
    obj.lvl.destroyObject(obj)
  else
    if obj.lvl.data.tick%4 == 0 then
      local dx = math.random(-8, 8)
      local dy = math.random(-8, 8)
      local flash = obj:spawn("animation", dx, dy, 10, "spark1")
      flash:setColor(255, 0, 0)
    end
    if obj.lvl.data.tick%20 == 0 then
      obj:playSound("shake")
    end
  end
end

return states