local animations =
{
  file = "spark",
  idle = { start = 1, frames = 4, cycle = "loop", steps = 6 }
}

local north = { 0, -1 }
local east = { 1, 0 }
local south = { 0, 1 }
local west = { -1, 0 }

local march =
{
  east, north, east, east,
  south, nil, south, south,
  west, north, nil, east,
  west, north, west, nil,
}

local parent = require("obj.agent")
local class = game.oo.class(parent)

class.nosleep = true
class.animations = animations

function class:init(lvl, x, y, r)
  parent.init(self, lvl, x, y, 8)
  local body = self.body
  body:setType("kinematic")
  body:setGravityScale(0)
  self.shape = body:newCircle(3)
  self.shape:setFilterData(game.cat.enemy, game.cat.player, -2)
  --self.shape:setSensor(true)
  local tx, ty = self:getTile()
  local bx, by = self:nextStep(tx, ty, east)
  self.reverse = r
  self:setAnimation(self.animations.idle)
  self:gotoNext()
end

function class:destroy()
  self:setAnimation()
  self.shape = nil
  self.targ = nil
  self.prev = nil
  self.reverse = nil
  parent.destroy(self)
end

function class:nextStep(ax, ay, p)
  -- adjacent
  local lvl = self.lvl
  local w, h = lvl.getDimensions()
  local map = lvl.map
  local i = (ay - 1)*w + ax
  local b = 0
  if not lvl.isWalkable(i) then
    b = b + 1
  end
  if not lvl.isWalkable(i + 1) then
    b = b + 2
  end
  if not lvl.isWalkable(i + w) then
    b = b + 8
  end
  if not lvl.isWalkable(i + w + 1) then
    b = b + 4
  end
  -- step
  if b == 5 then
    if p == east then
      b = 1
    else
      b = 4
    end
  elseif b == 10 then
    if p == south then
      b = 2
    else
      b = 8
    end
  end
  if not self.reverse then
    b = 15 - b
  end
  return march[b + 1] or south
end

function class:gotoNext()
  local tx, ty = self:getTile()
  local s = self:nextStep(tx, ty, self.prev or east)
  local dx, dy = unpack(s)
  self.targ = { tx + dx, ty + dy }
  self.prev = s
end

function class:update(dt)
  parent.update(self, dt)
  local lvl = self.lvl
  local q = 15
  local e = lvl.data.tick%(q + 1)
  --local q = 0.5
  --local e = self.elapsed + dt
  --if e < q then
  if e < q then
    local r = e/q
    local tx, ty = unpack(self.targ)
    local qx, qy = unpack(self.prev)
    local ox, oy = tx - qx, ty - qy
    local dx, dy = tx - ox, ty - oy
    local px, py = ox + dx*r, oy + dy*r
    self.body:setPosition(px*8, py*8)
  else
    self:gotoNext()
  end
  --self.elapsed = e%q
end

function class:collision(other, contact)
  other:call("hit", 1)
end

return class