local states = {}

states.spawn = {}
function states.spawn:update(obj, dt)
  obj.body:setActive(true)
  obj.spawns = obj.spawns + 1
  obj.health = 1
  obj.invincible = 0
  obj:setState("standing", 0)
end

states.standing = {}
function states.standing:enter(obj, wait)
  obj.wait = wait or math.random()*3 + 1
end
function states.standing:exit(obj)
  obj.wait = nil
end
function states.standing:update(obj, dt)
  if obj.elapsed >= obj.wait then
    local list = {}
    obj:detect(game.cat.player, list)
    local x1, y1 = obj:getPosition()
    for i = #list, 1, -1 do
      local x2, y2 = list[i]:getPosition()
      if obj:rayCast("wall", x1, y1, x2, y2) then
        table.remove(list, i)
      end
    end
    
    if #list > 0 then
      local other = list[1]
      obj.target = other
      obj.shots = math.random(1, 3)
      local x1, y1 = obj:getPosition()
      local x2, y2 = other:getPosition()
      local dx, dy = x2 - x1, y2 - y1
      local d = math.sqrt(dx*dx + dy*dy)
      obj.nx, obj.ny = dx/d, dy/d
      if x1 > x2 then
        obj:setState("swingleft")
      else
        obj:setState("swingright")
      end
    end
  end
end

states.swingleft = {}
function states.swingleft:enter(obj)
  obj:spawn("animation", -3, -2, 8, "chargeup")
  obj:playSound("chargeup")
end
function states.swingleft:update(obj, dt)
  if obj.elapsed >= 0.5 then
    obj:setState("fireleft")
  end
end

states.fireleft = {}
function states.fireleft:enter(obj)
  obj:spawn("egg", -4, 0, obj.nx, obj.ny, game.cat.player)
end
function states.fireleft:update(obj, dt)
  if obj.elapsed >= 0.2 then
    obj.shots = obj.shots - 1
    if obj.shots > 0 then
      obj:setState("fireleft")
    else
      obj:setState("standing")
    end
  end
end

states.swingright = {}
function states.swingright:enter(obj)
  obj:spawn("animation", 3, -2, 8, "chargeup")
  obj:playSound("chargeup")
end
function states.swingright:update(obj, dt)
  if obj.elapsed >= 0.5 then
    obj:setState("fireright")
  end
end

states.fireright = {}
function states.fireright:enter(obj)
  obj:spawn("egg", 4, 0, obj.nx, obj.ny, game.cat.player)
end
function states.fireright:update(obj, dt)
  if obj.elapsed >= 0.2 then
    obj.shots = obj.shots - 1
    if obj.shots > 0 then
      obj:setState("fireright")
    else
      obj:setState("standing")
    end
  end
end

states.dead = {}
function states.dead:enter(obj)
  obj.body:setActive(false)
end
function states.dead:awake(obj)
  obj:setState("spawn")
end

return states