local animations =
{
  file = "switch",
  left = { start = 1, frames = 3, cycle = "none" },
  right = { start = 1, frames = 3, cycle = "reverse" },
}

local parent = require("obj.agent")
local class = game.oo.class(parent)

class.animations = animations
class.nosleep = true

function class:init(lvl, x, y)
  parent.init(self, lvl, x, y)
  local fixture = self.body:newBox(2, 2, 0, 2)
  fixture:setFilterData(game.cat.wall, game.cat.default, 0)
  fixture:setSensor(true)
  self.touching = false
  self.body:setType("static")
  self.mode = false
  self.count = 0
end

function class:destroy()
  self.touching = nil
  parent.destroy(self)
end

function class:update(dt)
  parent.update(self, dt)
  local tile = self.tiles[1]
  local locked = tile:detect("dynamic", nil, tile.sensor)
  if not locked then
    local istouching = self:detect(game.cat.player)
    local wastouching = self.touching
    if istouching and not wastouching then
      local on = not self.mode
      self.count = self.count + 1
      if self.limit and self.count > self.limit then
        return
      end
      self:playSound(on and "switch2" or "switch")
      self:toggle(on)
      if self.tiles then
        for i, obj in ipairs(self.tiles) do
          obj:toggle(on)
        end
      end
      if self.switches then
        for i, obj in ipairs(self.switches) do
          if obj ~= self then
            obj:toggle(on)
          end
        end
      end
    end
    self.touching = istouching
  end
end

function class:toggle(on)
  local a = self.animations[on and "right" or "left"]
  self:setAnimation(a)
  self.mode = on
end

function class:setTarget(tiles, switches)
  self.tiles = tiles
  self.switches = switches
end

return class