local cat = game.cat
local states = {}

states.spawn = {}
function states.spawn:update(obj, dt)
  obj:reposition()
  obj.sprite:setRotation(0)
  obj.spawns = obj.spawns + 1
  obj.health = 1
  obj.invincible = 0
  local w = math.random(2, 8)
  obj:setState("disappear", w)
  obj.elapsed = 1
end

states.appear = {}
function states.appear:enter(obj)
  obj:playSound("tornado")
end
function states.appear:update(obj, dt)
  if obj.elapsed > obj.animation.total then
    obj.body:setActive(true)
    obj:setState("seek", 0)
    obj.path = nil
    obj.last = 0
  end
end

local deltas = { {-1,0}, {0,-1}, {1,0}, {0,1} }
states.seek = {}
function states.seek:enter(obj, wait)
  obj.wait = wait or obj.animations.seek.total
end
function states.seek:update(obj, dt)
  obj.last = obj.last + dt
  obj.wait = obj.wait - dt
  if obj.wait > 0 then
    return
  end
  local path = obj.path
  if not path or #path < 2 or obj.last > 3 then
    obj.path = nil
    if math.random(5) < 4 then
      local tx, ty = obj:getTile()
      local p = { x = tx, y = ty }
      obj.path = { p }
      obj.last = 0
      obj.wait = obj.animations.seek.total
      for i = 1, 16 do
        local d = math.random(1, 4)
        local dx, dy = unpack(deltas[d])
        local px, py = p.x + dx, p.y + dy
        if px ~= p.x or py ~= p.y then
          local lvl = obj.lvl
          if lvl.isWalkableXY(px, py) and lvl.getTile(px, py) ~= "block" then
            p = { x = px, y = py }
            table.insert(obj.path, p)
          end
        end
      end
    end
  else
    obj:setState("move")
  end
end

states.move = {}
function states.move:enter(obj)
  local q = obj.path[2]
  obj:turnTo(q.x, q.y)
end
function states.move:update(obj, dt)
  if obj.elapsed >= obj.animation.total then
    local q = obj.path[2]
    obj:moveTo(q.x, q.y)
    table.remove(obj.path, 1)
    if #obj.path == 0 then
      obj.path = nil
    end
    if math.random(1, 20) > 1 then
      obj:setState("seek")
    else
      obj:setState("disappear")
    end
  end
end

states.disappear = {}
function states.disappear:enter(obj, wait)
  obj.wait = wait or math.random(6, 20)
  obj.body:setActive(false)
end
function states.disappear:update(obj, dt)
  if obj.elapsed >= obj.wait then
    obj.elapsed = 1
    --[[
    obj.path = obj:findPlayer()
    if obj.path then
      obj:setState("appear")
    end
    ]]
    obj:setState("appear")
  end
end

states.dead = {}
function states.dead:update(obj, dt)
  obj.body:setActive(false)
  --if obj.elapsed >= obj.animation.total then
    --obj:setState("spawn")
  --end
end
function states.dead:awake(obj)
  obj:setState("spawn")
end

return states