local cat = game.cat
local animations =
{
  file = "tornado", ox = -4, oy = -12,
  seek = { start = 1, frames = 4, cycle = "loop", steps = 2 },
  move = { start = 5, frames = 5, cycle = "none", steps = 2 },
  dead = { start = 10, frames = 7, cycle = "none" },
  appear = { start = 10, frames = 7, cycle = "reverse", steps = 4 },
  disappear = { start = 10, frames = 7, cycle = "none" },
}

local parent = require("obj.agent")
local class = game.oo.class(parent)

class.states = require("obj.tornado-states")
class.animations = animations
--class.bonus = 300

function class:init(lvl, x, y)
  parent.init(self, lvl, x, y)
  local body = self.body
  body:setType("kinematic")
  body:setGravityScale(0)
  body:setActive(false)
  self.shape = body:newCircle(2.5)
  self.shape:setFilterData(cat.enemy, cat.player + cat.bullet, -2)
  self:setState("spawn")
end

function class:destroy()
  self.shape = nil
  --self.speed = nil
  parent.destroy(self)
end

function class:turnTo(x2, y2)
  local px, py = self:getPosition()
  local x1, y1 = math.ceil(px/8), math.ceil(py/8)
  local a = 0
  if x1 < x2 then
    a = math.pi/2
  elseif x1 > x2 then
    a = -math.pi/2
  elseif y1 < y2 then
    a = math.pi
  end
  self.sprite:setRotation(a)
end

function class:moveTo(x2, y2)
  local px, py = self:getPosition()
  local x1, y1 = math.ceil(px/8), math.ceil(py/8)
  if x1 == x2 and y1 == y2 then
    return
  end
  self.elapsed = 0
  local x3, y3 = x2*8 - 4, y2*8 - 4
  self.body:setAwake(true)
  self.body:setPosition(x3, y3)
end

--[[
function class:findPlayer()
  local lvl = self.lvl
  local d = lvl.data
  local p = d.p1 or d.p2
  if p then
    local x1, y1 = self:getPosition()
    local x2, y2 = p:getPosition()
    local dx, dy = x2 - x1, y2 - y1
    local d = math.sqrt(dx*dx + dy*dy)
    if d < 8*16 then
      return self:findPath(x2, y2)
    end
  end
end
]]

function class:onDamage(h)
  if h == 0 then
    self:playSound("shake")
    game.screenshake(self.lvl, 0.25)
  end
end

function class:onDie(pts)
  --self:dropBonus()
  self:setState("dead")
end

function class:collision(other, contact)
  if self.health > 0 then
    other:call("hit", 1)
    self:damage()
  end
end

function class:shot(pts)
  self:damage(pts)
end

return class