local cat = game.cat
local filters =
{
  wall = { [cat.wall] = true },
  env = { [cat.wall] = true, [cat.platform] = true },
  player = { [cat.player] = true },
  enemy = { [cat.enemy] = true },
  laser = { [cat.wall] = true, [cat.player] = true },
}

local query = {}

function query.raycast(fixture, x, y, xn, yn, ratio)
  local c = fixture:getFilterData()
  if fixture:isSensor() or not filters[query.filter][c] then
    return -1
  end
  if query.found == nil or ratio < query.ratio then
    query.found = fixture
    query.x = x
    query.y = y
    query.ratio = ratio
  end
  return ratio
end

function query.pointquery(fixture)
  local c = fixture:getFilterData()
  if fixture:isSensor() or not filters[query.filter][c] then
    return true
  end
  local shape = fixture:getShape()
  local kind = shape:getType()
  if kind ~= "circle" and kind ~= "polygon" then
    return true
  end
  query.found = fixture
  return false
end

local parent = require("obj.body")
local class = game.oo.class(parent)

function class:init(lvl, x, y, z)
  parent.init(self, lvl, x, y, z or 1)
  self.detected = 0
end

function class:destroy()
  self.detected = nil
  parent.destroy(self)
end

function class:setHitbox(...)
  if self.hitbox then
    self.hitbox:destroy()
    self.hitbox = nil
  end
  self.hitbox = self.body:newBox(...)
  self.hitbox:setFilterData(game.cat.wall, game.cat.default, 0)
  self.hitbox:setSensor(true)
end

function class:rayCast(q, x1, y1, x2, y2)
  local dx, dy = x2 - x1, y2 - y1
  if dx*dx + dy*dy <= 1 then
    return
  end
  query.found = nil
  query.filter = q
  self.lvl.world:rayCast(x1, y1, x2, y2, query.raycast)
  if query.found then
    local body = query.found:getBody()
    return body:getUserData(), query.x, query.y
  end
end

function class:pointQuery(q, x, y)
  query.found = nil
  query.filter = q
  self.lvl.world:queryBoundingBox(x, y, x, y, query.pointquery)
  if query.found then
    local body = query.found:getBody()
    return body:getUserData()
  end
end

function class:detect(cat, list, body)
  body = body or self.body
  local contacts = body:getContacts()
  for i, contact in ipairs(contacts) do
    if contact:isTouching() and contact:isEnabled() then
      local f1, f2 = contact:getFixtures()
      local other = f2:getBody()
      if other == body then
        other = f1:getBody()
        f1, f2 = f2, f1
      end
      if not f2:isSensor() then
        if cat == nil or cat == other:getType() or cat == f2:getFilterData() then
          local obj = other:getUserData()
          if list then
            table.insert(list, obj)
          else
            return obj
          end
        end
      end
    end
  end
  return list
end

function class:update(dt)
  local list = {}
  self:detect(game.cat.player, list)
  local n = #list
  if n ~= self.detected then
    self.detected = n
    self:call("onDetect", n)
  end
end

return class