local parent = require("obj.wall")
local class = game.oo.class(parent)
local cat = game.cat

function class:init(lvl, x, y, bits)
  parent.init(self, lvl, x, y, bits, 4)
  self.type = "wall"
  self:setGraphic("wallb", bits + 1)
end

function class:setMoving(path, speed)
  assert(not self.path)
  self.body:setType("kinematic")
  self.time = 0
  self.total = 0
  for j, pt in ipairs(path) do
    local dx, dy = unpack(pt)
    local d = math.sqrt(dx*dx + dy*dy)
    pt.dist = d
    self.total = self.total + d
  end
  self.path = path
  self.speed = speed
  local x, y = self:getPosition()
  self.tx = math.ceil(x/8)
  self.ty = math.ceil(y/8) - 1
  self.update = self.updateMoving
end

function class:updateMoving(dt)
  local path = self.path
  if path then
    local d = self.lvl.data
    local e = self.time + d.elapsed
    local q = e*self.speed
    q = q%self.total
    local total = 0
    local index = 1
    local dx, dy = 0, 0
    for j, pt in ipairs(path) do
      if total + pt.dist >= q then
        index = j
        break
      end
      total = total + pt.dist
      dx = dx + pt[1]
      dy = dy + pt[2]
    end
    local found = path[index]
    local ratio = (q - total)/found.dist
    local x = self.tx + dx + found[1]*ratio
    local y = self.ty + dy + found[2]*ratio
    self:movePlatform(x*8 - 4, y*8 + 4, self.up)
  end
end

function class:setSensor(s)
  if self.sensor ~= s then
    self.sensor = s
    local q = (8 - 1)/2
    local x, y = self:getPosition()
    local f = s:newBox(q, q, x, y)
    f:setFilterData(cat.wall, cat.default, 0)
    f:setSensor(true)
  end
end

function class:setDisappearing(period, d, t)
  self.period = period
  self.dutycycle = d
  self.time = t or 0
  self.body:setType("kinematic")
  self.update = self.updateDisappearing
end

function class:updateDisappearing(dt)
  local sensor = self.sensor
  if sensor then
    local d = self.lvl.data
    local e = self.time + d.elapsed
    local p = self.period
    local t = (e + p/2)%p
    local on = t > p*self.dutycycle
    local body = self.body
    if on and not body:isActive() then
      if self:detect("dynamic", nil, sensor) then
        on = false
      else
        self:playSound("hiss")
      end
    end
    self:toggle(on)
  end
end

return class