local cat = game.cat
local animations =
{
  file = "worm", ox = -4, oy = -4,
  move = { start = 1, frames = 16, cycle = "none", steps = 1 },
  appear = { start = 17, frames = 11, cycle = "reverse", steps = 4 },
  disappear = { start = 17, frames = 11, cycle = "none" },
  dead = { start = 17, frames = 11, cycle = "none" },
}

local normals =
{
  { 0, -1 },
  { -1, 0 },
  { 1, 0 },
  { 0, 1 },
}

local parent = require("obj.agent")
local class = game.oo.class(parent)

class.states = require("obj.worm-states")
class.animations = animations
class.nosleep = true

function class:init(lvl, x, y, nx, ny, mask)
  mask = mask or cat.player
  parent.init(self, lvl, x, y)
  local body = self.body
  body:setType("kinematic")
  body:setGravityScale(0)
  body:setActive(false)
  self.shape = body:newCircle(2.5)
  self.shape:setFilterData(cat.enemy, cat.bullet + mask, -2)
  self.path = { nx, ny }
  self.lockout = -1
  self.time = 0
  self:setState("spawn")
end

function class:destroy()
  self.shape = nil
  self.path = nil
  self.lockout = nil
  self.time = nil
  parent.destroy(self)
end

function class:turnTo(x2, y2)
  local px, py = self:getPosition()
  local x1, y1 = math.ceil(px/8), math.ceil(py/8)
  local a = 0
  if x1 < x2 then
    a = math.pi/2
  elseif x1 > x2 then
    a = -math.pi/2
  elseif y1 < y2 then
    a = math.pi
  end
  self.sprite:setRotation(a)
end

function class:moveTo(x2, y2)
  local px, py = self:getPosition()
  local x1, y1 = math.ceil(px/8), math.ceil(py/8)
  if x1 == x2 and y1 == y2 then
    return
  end
  self.elapsed = 0
  local x3, y3 = x2*8 - 4, y2*8 - 4
  self.body:setAwake(true)
  self.body:setPosition(x3, y3)
end

function class:onDie(pts)
  self:setState("dead")
  if self.multiply then
    self:playSound("jump2")
    for i, v in ipairs(normals) do
      self:spawn("worm", 0, 0, v[1], v[2])
    end
  else
    self:playSound("shake")
    --game.screenshake(self.lvl, 0.25)
  end
end

function class:collision(other, contact)
  other:call("hit", 1)
  self:damage()
  self.multiply = true
end

function class:shot(pts)
  self:damage(pts)
  self.multiply = true
end

return class