local parent = require("states.screen")
local class = parent:class()

local kcodes = require("defs.keycodes")
local jcodes = require("defs.joycodes")

function class:create(d, a)
  parent.create(self)
  self.time = 0
  self.sprite = self.layer:newSprite(-100, -4)
  self.text = love.graphics.newText(app.fonts.mini)
  self.sprite:setGraphic(self.text)
  self.device = d
  self.action = a
  self.poll = {}
end

function class:destroy()
  self.sprite = nil
  self.text = nil
  self.time = nil
  self.device = nil
  self.action = nil
  self.poll = nil
  parent.destroy(self)
end

function class:draw()
  self.view:draw()
  local a = app.lang.format(app.lang.press, app.lang[self.action])
  if self.time%2 > 1 then
    a = self.device
  end
  self.text:setf(a, 200, "center")
end

function class:update(dt)
  self.time = self.time + dt

  local active = {}
  local d = app.attached[self.device]
  if d then
    local c = jcodes
    if d == love.keyboard then
      c = kcodes
    end
    for _, v in ipairs(c) do
      local s = app.input.isDown(d, v)
      if s == true and self.poll[v] == false then
        table.insert(active, v)
      end
      self.poll[v] = s
    end
  end
  
  if self.time > 10 then
    self:done(false)
  elseif #active > 0 then
    self.raw = active[1]
  elseif #active == 0 then
    if self.raw then
      local raw = self.raw -- r[1]
      local bind = app.bindings[self.device]
      for k, v in pairs(bind) do
        if v == self.action then
          bind[k] = nil
        end
      end
      bind[raw] = self.action
      self:done(true)
    end
  end
end

function class:done(e)
  app.playSound(e and "activate" or "error")
  app.popstate()
  app.callstate("reset")
end

function class:cancel()
  self:done(false)
end

return class