local parent = require("states.menu")
local class = parent:class()

local line = 16
local red = {255, 0, 0}
local white = {255, 255, 255}
local black = {0, 0, 0}

function class:create(...)
  parent.create(self, ...)
end

function class:destroy()
  parent.destroy(self)
end

function class:layout(n, ...)
  local fn = "layouts."..n
  package.loaded[fn] = nil
  local def = require(fn)

  self.def = def
  self:event("init", ...)
  def.selected = def.selected or 1
  
  local layer = self.layer
  layer:destroyChildren()
  local w, h = self.view:getDimensions()
  local side = 24
  local rw, rh = w/app.s, h/app.s
  local c = def.cols
  local r = def.rows
  c = c or math.floor(rw/side)
  r = r or math.floor(rh/side)
  local bw = (rw - line)/c
  local bh = (rh - line*2)/r
  if not def.cols or not def.rows then
    local s = math.min(bw, bh)
    bw, bh = s, s
  end
  bw = math.ceil(bw)
  bh = math.ceil(bh)
  def.cols = c
  def.rows = r
  def.bw = bw
  def.bh = bh

  local rr = math.ceil(#def/c)
  rr = math.min(rr, r)
  while #def > c*r do
    table.remove(def)
  end

  local ox = bw*(c - 1)/2
  local oy = bh*(rr - 1)/2 - line/2
  local hbw, hbh = bw/2, bh/2
  local nbw, nbh = hbw - 2, hbh - 2
  local verts1 = { {-hbw,-hbh},{hbw,-hbh},{hbw,hbh},{-hbw,hbh} }
  local verts2 = { {-nbw,-nbh},{nbw,-nbh},{nbw,nbh},{-nbw,nbh} }
  local border = love.graphics.newMesh(verts1, "fan", "static")
  local bg = love.graphics.newMesh(verts2, "fan", "static")
  self.borders = {}
  self.bg = {}
  self.fg = {}
  for i, v in ipairs(def) do
    local tx = (i - 1)%c
    local ty = (i - tx - 1)/c
    local x = tx*bw - ox
    local y = ty*bh - oy
    local txt = love.graphics.newText(app.fonts.mini)
    if v.text then
      --txt:setf(v.text, bw - line/2, "left")
      txt:set(v.text)
    end
    local s0 = layer:newSprite(x, y)
    local s1 = layer:newSprite(x, y)
    local s2 = layer:newSprite(x, y)
    s0:setGraphic(border)
    s1:setGraphic(bg)
    s2:setGraphic(txt, -bw/2 + line/4, -bh/2 + line/4)
    table.insert(self.borders, s0)
    table.insert(self.bg, s1)
    table.insert(self.fg, s2)
  end

  if def.title then
    local txt = love.graphics.newText(app.fonts.mini, def.title)
    local fw = app.fonts.mini:getWidth(def.title)
    self.title = layer:newSprite(0, -bh*rr/2 - line/2)
    self.title:setGraphic(txt, -fw/2, 0)
  end
  
  self.elapsed = 0
end

function class:update(dt)
  self:event("update", dt)
  self.elapsed = self.elapsed + dt
  if self.previous ~= self.def.selected then
    self.previous = self.def.selected
    self.elapsed = 0
  end
  
  local def = self.def
  local bw, bh = def.bw, def.bh
  for i, v in ipairs(def) do
    local r = 1
    local s = self.bg[i]
    local b = self.borders[i]
    if v.img then
      local iw, ih = v.img:getDimensions()
      local sx, sy = (bw - 4)/iw, (bh - 4)/ih
      s:setGraphic(v.img, -iw/2*sx, -ih/2*sy, 0, sx, sy)
      s:setColorRGB(v.color or white)
      b:setColorRGB(i == def.selected and red or black)
    else
      local c = v.color or black
      if i == def.selected then
        r = math.min(self.elapsed/0.2, 1)
        if r == 1 then
          c = red
        end
      end
      s:setScale(1, r)
      s:setColorRGB(c)
      b:setColorRGB(white)
    end
    local f = self.fg[i]
    f:setVisible(r == 1)
    local a = v.disabled and 0.5 or 1
    b:setAlpha(a)
    f:setAlpha(a)
  end
end

function class:up()
  self:select(-self.def.cols)
end

function class:down()
  self:select(self.def.cols)
end

function class:left()
  self:select(-1)
end

function class:right()
  self:select(1)
end

return class