local parent = require("states.screen")
local class = parent:class()

function class:create(s)
  parent.create(self)
  self:restart(s)
end

function class:destroy()
  app.stopMusic()
  local lvl = self.lvl
  game.flush(lvl)
  game.unload(lvl)
  -- send stats
  if app.user then
    game.progress.send(app.user)
  end
  game.progress.export()
  self.lvl = nil
  self.accum = nil
  self.ready = nil
  parent.destroy(self)
end

function class:update(dt)
  if not self.ready then
    self.ready = true
    return
  end
  -- update with accumulator
  local lvl = self.lvl
  local d = lvl.data
  local it = 1/app.ops.framerate
  self.accum = math.min(self.accum + dt, it*app.ops.frameskip)
  local steps = 0
  while self.accum and self.accum > it do
    self.accum = self.accum - it
    game.update(lvl, it)
    steps = steps + 1
  end
  if d.over then
    d.over = d.over - it*steps
    if d.over <= 0 then
      local session = lvl.session
      local socket = game.socket
      if d.complete then
        -- mission complete
        local cheat = d.cheat
        local title = lvl.title
        local ms = d.elapsed*1000
        local nextup = tonumber(title) + 1
        nextup = tostring(nextup)
        if app.exists("/lvl/"..nextup..".png") then
          -- next arena
          session.level = nextup
          if socket then
            self:restart(session)
          else
            app.popstate()
            app.pushstate("states.grid", "levels", session)
          end
        else
          -- finished
          app.popstate()
          app.pushstate("states.title")
          if app.exists("/lvl/"..nextup..".lua") then
            app.pushstate("states.buy")
          end
        end
        -- upload hiscore
        if not socket and not cheat then
          if app.exists("/states/hiscore.lua") then
            if app.portal and app.portal.isConnected() then
              app.pushstate("states.hiscore", title, ms)
            end
          end
        end
      else
        -- game over
        app.popstate()
        if socket then
          -- back to lobby
          app.pushstate("states.lobby", session)
        else
          -- back to title
          app.pushstate("states.title")
        end
      end
    end
  end
end

function class:restart(session)
  app.stopMusic()
  local socket = game.socket
  local lvl = self.lvl
  if lvl then
    game.unload(lvl)
  end
  game.socket = socket
  session.lives = math.max(session.lives, game.minlives)
  lvl = game.load(session)
  self.lvl = lvl
  self.ready = false
  self.accum = 0
  local track = lvl.def.music
  if track then
    app.playMusic(track)
  end
end

function class:draw()
  game.draw(self.lvl)
end

function class:detach(id)
  local s = self.lvl.session
  if s.p1c == id then
    s.p1c = nil
    self:start()
  elseif s.p2c == id then
    s.p2c = nil
    self:start()
  end
end

function class:pause()
  if game.socket or self.lvl.data.over then
    return
  end
  app.playSound("pause")
  app.pushstate("states.menu", "pause")
  self.lvl.broadcast("pause")
end

function class:resume()
  self.lvl.broadcast("resume")
end

function class:start()
  self:pause()
end

function class:cancel()
  self:pause()
end

function class:left(id)
  if app.arg.devmode and not game.socket then
    if id == "Keyboard" and love.keyboard.isDown("lctrl") then
      local lvl = self.lvl
      local p1 = lvl.data.p1
      local cp = p1 and p1.checkpoint
      if p1 and cp.ident > 1 then
        lvl.data.cheat = true
        lvl.session.lives = lvl.session.lives + 1
        p1.checkpoint = lvl.checkpoint[cp.ident - 1]
        p1:damage(1)
      end
    end
  end
end

function class:right(id)
  if app.arg.devmode and not game.socket then
    if id == "Keyboard" and love.keyboard.isDown("lctrl") then
      local lvl = self.lvl
      local p1 = lvl.data.p1
      local cp = p1 and p1.checkpoint
      if p1 and cp.ident < #lvl.checkpoint then
        lvl.data.cheat = true
        lvl.session.lives = lvl.session.lives + 1
        p1.checkpoint = lvl.checkpoint[cp.ident + 1]
        p1:damage(1)
      end
    end
  end
end

return class