local parent = require("states.screen")
local class = parent:class()

function class:create()
  parent.create(self)
  local img = love.graphics.newImage("/gfx/steam.png")
  local iw, ih = img:getDimensions()
  img:setFilter(app.ops.filter, app.ops.filter)
  self.title = self.layer:newSprite(0, 0)  
  self.title:setAlpha(0)
  self.title:setGraphic(img, -iw/2, -ih/2)
  self.time = 0
  
  if not app.arg.standalone and app.exists("/utils/sworks/main.lua") then
    local portal = require("utils.sworks.main")
    if app.pcall(portal.init) then
      app.portal = portal
      local user = portal.getUser()
      if user then
        user:requestStats(function(ok)
          if ok then
            -- associate progress file with userid
            local uid = user:getId()
            print("profile:"..uid)
            app.ops.user = uid
            -- remember user profile
            app.user = user
            
            -- language preference from the client
            local pref = portal.getLanguage()
            if pref and pref ~= app.ops.preferred then
              app.ops.preferred = pref
              local lc = app.ops.language
              if lc ~= "none" and lc ~= pref then
                if app.text.locales[pref] then
                  print("locale override:"..pref)
                  app.ops.language = pref
                  app.lang = app.text.locales[pref]
                end
              end
            end
          end
        end)
      end
    end
  end
end

function class:destroy()
  self.title = nil
  self.time = nil
  parent.destroy(self)
end

function class:update(dt)
  self.time = self.time + dt

  local portal = app.portal
  local user = app.user
  if (not portal) or (portal.isRunning() and user) then
    if app.ops.os ~= "Web" then
      game.progress.import()
    end
    if user then
      game.progress.receive(user)
    end
    app.popstate()

    -- player invite?
    for i, v in ipairs(arg) do
      local id = arg[i + 1]
      if v == "+connect_lobby" and id and portal then
        local session = game.newSession()
        session.lobby = id
        app.pushstate("states.lobby", session)
        return
      end
    end

    app.pushstate("states.title")
    if app.ops.language == "none" and table.count(app.text.locales) > 1 then
      app.pushstate("states.menu", "lang")
    end
  end
end

function class:draw()
  --love.graphics.reset()
  --love.graphics.setColor(1,0,0,1)
  --love.graphics.rectangle('fill',0,0,100,100)
  if self.time > 3 then
    local a = math.sin(self.time%3/3*math.pi)
    self.title:setAlpha(a)
  end
  self.view:draw()
end

return class