local lg = love.graphics

local parent = require("states.state")
local class = parent:class()

function class:create()
  parent.create(self)
  print("splash started")
  self.time = 0
  self.logo = love.graphics.newImage("/gfx/logo.png")
  self.logo:setFilter(app.ops.filter, app.ops.filter)
  local w, h = self.logo:getDimensions()
  self.data = love.image.newImageData(w, h)
  self.unvisited = {}
  for y = 0, h - 1 do
    table.insert(self.unvisited, y)
    for x = 0, w - 1 do
      self.data:setPixel(x, y, 0, 0, 0, 1)
    end
  end
  self.cover = love.graphics.newImage(self.data)
  self.cover:setFilter(app.ops.filter, app.ops.filter)
  if app.ops.shader ~= "none" then
    local fn = "shaders/"..app.ops.shader..".frag"
    if app.exists(fn) then
      app.shaders = {}
      app.shaders.default = love.graphics.newShader(fn)
      self.canvas = love.graphics.newCanvas()
    end
  end
  love.mouse.setVisible(not app.ops.fullscreen)
end

function class:destroy()
  self.time = nil
  self.logo = nil
  self.data = nil
  self.unvisited = nil
  self.cover = nil
  parent.destroy(self)
  print("splash complete")
end

function class:update(dt)
  dt = math.min(dt, 1/60)
  self.time = self.time + dt
  
  if self.time == dt then
    if love.audio then
      local src = love.audio.newSource("/sfx/boot.flac", "stream")
      src:setVolume(app.ops.mvolume)
      src:play()
      self.boot = src
    end
  elseif self.time >= 0.25 and not self.revealed then
    local w, h = self.cover:getDimensions()
    local j = math.random(1, #self.unvisited)
    local y = table.remove(self.unvisited, j)
    for x = 0, w - 1 do
      local i = y*w + x + 1
      self.data:setPixel(x, y, 0, 0, 0, 0)
    end
    self.revealed = #self.unvisited == 0
    
    self.cover = love.graphics.newImage(self.data)
    self.cover:setFilter(app.ops.filter, app.ops.filter)
  elseif self.revealed and not self.loaded then
    if not self.co then
      local co = require("core.loader")
      self.co = coroutine.create(co)
    end
    local ok, res = coroutine.resume(self.co)
    assert(ok, res)
    self.loaded = res == -1
  elseif self.loaded then
    self.wait = (self.wait or 0) + dt
    if self.wait >= 2 then
      if love.audio then
        if self.boot then
          self.boot:stop()
          self.boot = nil
        end
        local src = love.audio.newSource("/sfx/beep.flac", "stream")
        src:setVolume(app.ops.mvolume)
        src:play()
      end
      app.popstate()
      app.pushstate("states.portal")
    end
  end
end

function class:draw()
  local s = app.s
  local w, h = love.graphics.getDimensions()
  local iw, ih = self.logo:getDimensions()
  if s > 1 and s*iw > app.w then
    s = s/2
  end
  lg.reset()
  if self.canvas then
    lg.setCanvas(self.canvas)
    lg.clear(0, 0, 0, 1)
  end
  lg.draw(self.logo, w/2 - iw/2*s, h/2 - ih/2*s, 0, s, s)
  lg.draw(self.cover, w/2 - iw/2*s, h/2 - ih/2*s, 0, s, s)
  if app.fonts then
    lg.setFont(app.fonts.mini)
    local txt = app.ops.version.." "..app.ops.build
    lg.print(txt, 10, 10, 0, 2, 2)
    local v1, v2, v3, v4 = love.getVersion()
    local ver = v1.."."..v2.."."..v3.." "..v4
    local ww = app.fonts.mini:getWidth(ver)
    lg.print(ver, w - 10 - ww*2, 10, 0, 2, 2)
  end
  lg.setCanvas()
  if app.shaders and app.shaders.default and self.canvas then
    lg.setShader(app.shaders.default)
    lg.draw(self.canvas)
    lg.setShader()
  end
end

return class