local ops = app.ops
local shine = love.graphics.newImage("/gfx/shine.png")
local title = love.graphics.newImage("/gfx/title.png")
shine:setWrap("repeat", "repeat")
shine:setFilter(ops.filter, ops.filter)
title:setFilter(ops.filter, ops.filter)

local parent = require("states.screen")
local class = parent:class()

local verts = { {-150,-150},{150,-150},{150,150},{-150,150} }
local rect = love.graphics.newMesh(verts, "fan", "static")

function class:create()
  parent.create(self)
  local layer = self.layer
  self.bg = layer:newSprite(0, 0)
  self.bg:setGraphic(rect)
  self.bg:setColorRGB(255, 255, 0)
  local iw, ih = shine:getDimensions()
  self.quad = love.graphics.newQuad(0, 0, iw, ih*4, iw, ih)
  self.shine = layer:newSprite(0, 0)
  self.shine:setGraphic(shine, self.quad, -iw/2, -ih*2)
  self.shine:setColorRGB(255, 255, 255)
  local iw, ih = title:getDimensions()
  self.title = layer:newSprite(0, 0)
  self.title:setGraphic(title, -iw/2, -ih/2)
  self.title:setScale(2, 2)
  
  local fg = layer:newSprite(0, 0)
  function fg.draw(fg)
    local view = self.view
    local sx, sy = view:getScale()
    local vw, vh = view:getDimensions()
    vw, vh = vw/sx, vh/sy
    -- crop edges
    local lg = love.graphics
    lg.setColor(0, 0, 0, 1)
    lg.rectangle("fill", -vw/2, -vh/2, vw, vh/8)
    lg.rectangle("fill", -vw/2, vh/2 - vh/8, vw, vh/8)
  end
  
  local mini = app.fonts.mini
  local text = love.graphics.newText(mini)
  text:setf(app.lang.cap, 300, "center")
  self.caption = layer:newSprite(-150, 40)
  self.caption:setGraphic(text)
  local text2 = love.graphics.newText(mini)
  text2:setf(app.lang.rights, 300, "center")
  self.rights = layer:newSprite(-150, 50)
  self.rights:setGraphic(text2)
  local text3 = love.graphics.newText(mini)
  text3:setf(app.lang.welcome, 300, "center")
  self.welcome = layer:newSprite(-150, 50)
  self.welcome:setGraphic(text3)
  self.elapsed = 0
  
  app.playMusic("boss_cutscene")
end

function class:destroy()
  self.elapsed = nil
  self.bg = nil
  self.shine = nil
  self.title = nil
  parent.destroy(self)
end

function class:update(dt)
  self.elapsed = self.elapsed + dt
  local x, y = self.shine:getPosition()
  local p = 6
  local t = (self.elapsed + p/2)%p
  y = t/p*700 -- update sprite position
  local rx = math.floor(x + 0.5)
  local ry = math.floor(y + 0.5)
  self.shine:setPosition(rx, ry - 350)
  local on = self.elapsed%4 <= 2
  self.welcome:setVisible(on)
  self.rights:setVisible(not on)
  if self.elapsed >= 1 and self.elapsed - dt < 1 then
    app.say("welcome")
  end
end

function class:reset()
  self.elapsed = 0
end

function class:start()
  app.playSound("start")
  app.pushstate("states.menu", "main")
end

function class:shoot()
  self:start()
end

function class:cancel()
  class:start()
end

return class