local reg = debug.getregistry()
local lg = love.graphics

-- World
function reg.World:draw()
  local bodies = self:getBodies()
  for _, v in ipairs(bodies) do
    v:draw()
  end
  lg.setColor(1,0,0,1)
  local joints = self:getJoints()
  for _, joint in ipairs(joints) do
    local x1, y1, x2, y2 = joint:getAnchors()
    if joint.getGroundAnchors then
      local x3, y3, x4, y4 = joint:getGroundAnchors()
      lg.line(x1, y1, x3, y3, x4, y4, x2, y2)
    else
      lg.line(x1, y1, x2, y2)
    end
  end
end

-- Body
function reg.Body:draw()
  local x, y = self:getPosition()
  local r = self:getAngle()
  lg.push()
  lg.translate(x, y)
  lg.rotate(r)
  local fixtures = self:getFixtures()
  for _, fixture in ipairs(fixtures) do
    local shape = fixture:getShape()
    shape:draw()
  end
  lg.pop()
end

-- Shape
function reg.CircleShape:draw()
  local x, y = self:getPoint()
  local r = self:getRadius()
  lg.circle("line", x, y, r, 32)
  lg.line(x, y, x + r, y)
end

function reg.PolygonShape:draw()
  lg.polygon("line", self:getPoints()) 
end

function reg.ChainShape:draw()
  lg.line(self:getPoints()) 
end

function reg.EdgeShape:draw()
  lg.line(self:getPoints()) 
end