-- Combined callbacks for mice and touchscreens

local input = {}

local _tx = {}
local _ty = {}

-- mouse input

local function _press(id, x, y, p)
  _tx[id] = x
  _ty[id] = y
  input.press(id, x, y)
end

local function _release(id, x, y, p)
  input.release(id, x, y)
  _tx[id] = nil
  _ty[id] = nil
end

local function _move(id, x, y)
  _tx[id] = x
  _ty[id] = y
  input.move(id, x, y)
end

if love.touch then
  function love.touchpressed(id, x, y, dx, dy, p)
    _press(id, x, y)
  end
  function love.touchreleased(id, x, y, dx, dy, p)
    _release(id, x, y)
  end
  function love.touchmoved(id, x, y, dx, dy, p)
    _move(id, x, y)
  end
end
if love.mouse then
  local buttons = { "left", "right", "middle" }
  local rbuttons = { left = 1, right = 2, middle = 3 }
  function love.mousepressed(x, y, id, it)
    if not it then
      local b = buttons[id] or id
      _press(b, x, y)
    end
  end
  function love.mousereleased(x, y, id, it)
    if not it then
      local b = buttons[id] or id
      _release(b, x, y)
    end
  end
  function love.mousemoved(x, y, dx, dy, it)
    if not it then
      for b in pairs(_tx) do
        local id = rbuttons[b]
        if id then
          _move(b, x, y)
        end
      end
    end
  end
end

function input.position(b)
  if b == "mouse" then
    if love.mouse then
      return love.mouse.getPosition()
    end
  end
  return _tx[b], _ty[b]
end

-- callbacks
function input.press(id, x, y)
end
function input.release(id, x, y)
end
function input.move(id, x, y)
end

return input