local sprite = {}
local spriteMT = { __index = sprite }

local lg = love.graphics

local reg = debug.getregistry()
reg.Sprite = sprite

setmetatable(sprite, { __index = reg.Node })

function sprite.new(x, y)
  local t = reg.Node.new(x, y, spriteMT)
  t.graphic = love.math.newTransform()
  t.color = { 1, 1, 1, 1 }
  t.mode = "alpha"
  return t
end

function sprite:destroy()
  self.graphic = nil
  self.color = nil
  self.mode = nil
  self.img = nil
  self.quad = nil
  reg.Node.destroy(self)
end

-- Rendering
function sprite:setGraphic(img, quad, ...)
  self.img = img
  local graph = self.graphic
  if type(quad) == "userdata" then
    self.quad = quad
    graph:setTransformation(...)
  else
    self.quad = nil
    graph:setTransformation(quad, ...)
  end
  self.changed = true
end

function sprite:getGraphic()
  return self.img, self.quad
end

function sprite:setMode(mode)
  self.mode = mode
end

function sprite:getMode(mode)
  return self.mode
end

function sprite:getColor()
  local c = self.color
  return c[1], c[2], c[3]
end

function sprite:setColor(r, g, b)
  if type(r) == "table" then
    r, g, b = unpack(r)
  end
  local c = self.color
  c[1], c[2], c[3] = r, g, b
end

function sprite:setColorRGB(r, g, b)
  if type(r) == "table" then
    r, g, b = unpack(r)
  end
  local c = self.color
  c[1], c[2], c[3] = r/255, g/255, b/255
end

function sprite:getColorRGB()
  local c = self.color
  return c[1]*255, c[2]*255, c[3]*255
end

function sprite:getAlpha()
  return self.color[4]
end

function sprite:setAlpha(a)
  self.color[4] = a
end

-- Rendering
function sprite:setShader(s)
  self.shader = s
end

function sprite:draw()
  if not self.visible then
    return
  end
  local trans = self.transform
  if self.changed then
    trans:setTransformation(self.x, self.y, self.r, self.sx, self.sy)
    trans:apply(self.graphic)
    self.changed = nil
  end
  local img = self.img
  if not img then
    return
  end
  lg.setColor(self.color)
  lg.setBlendMode(self.mode)
  local quad = self.quad
  local shade = self.shader
  if shade then
    lg.setShader(shade)
    if quad then
      lg.draw(img, quad, trans)
    else
      lg.draw(img, trans)
    end
    lg.setShader()
  else
    if quad then
      lg.draw(img, quad, trans)
    else
      lg.draw(img, trans)
    end
  end
end

return sprite.new